//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "NNTPMods2.h"
#include "openssl.h"
#include "global.h"
#include "utility.h"
#include "qsadbs.h"
#include "msgmans.h"
#include "sha256.h"
#include "esub.h"
#include "sampledlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "IdExplicitTLSClientServerBase"
#pragma link "IdIOHandlerStack"
#pragma link "IdSSL"
#pragma link "IdCustomTransparentProxy"
#pragma link "IdExplicitTLSClientServerBase"
#pragma link "IdIOHandlerStack"
#pragma link "IdExplicitTLSClientServerBase"
#pragma link "IdIOHandlerStack"
#pragma link "IdSSL"
#pragma resource "*.dfm"
TNNTPMod2 *NNTPMod;
int NNTPThrdHandle = NULL;
TNNTPThread *NNTPThrd = NULL;
HANDLE NNTPModHandle = NULL;
HANDLE MainFormHandle = NULL;
//const AnsiString ignoreConnectErrors =
//"\"Connection Closed Gracefully.\","
//"\"Connection timed out.\","
//"\"Request rejected or failed.\"";
//---------------------------------------------------------------------------
// TNNTPThread
//---------------------------------------------------------------------------

void __fastcall GetNNTPMail(TForm *Owner, const AnsiString &account, int mode) //bool subsonly, bool catchup)
{
   if (NNTPThrd)
	  return;
   NNTPThrd = new TNNTPThread(Owner, account, mode);
}
//---------------------------------------------------------------------------

__fastcall TNNTPThread::TNNTPThread(bool CreateSuspended)
   : TThread(false)
{
   NNTPThrdHandle = Handle;
}
//---------------------------------------------------------------------------

__fastcall TNNTPThread::TNNTPThread(TForm *Owner, const AnsiString &Account, int Mode)
   : TThread(false)
{
   mode = Mode;
   MainFormHandle = Owner->Handle;
   NNTPThrdHandle = Handle;
   NNTPMod2 = new TNNTPMod2(Owner);
   NNTPModHandle = NNTPMod2->Handle;
   NNTPMod2->account = Account;
   DB->LoadAccount(Account, NNTPMod2->profile);
}
//---------------------------------------------------------------------------

__fastcall TNNTPThread::~TNNTPThread()
{
   NNTPModHandle = NULL;
   delete NNTPMod2;
   NNTPMod = NULL;
   NNTPThrdHandle = NULL;
   NNTPThrd = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPThread::Execute()
{
   switch (mode)
   {
	  case modeCatchUp: NNTPMod2->CatchUpGroup(); break;
	  case modeChkMail: NNTPMod2->CheckForMail(); break;
	  case modeGetMail: NNTPMod2->GetTheMail(); break;
   }
}

//---------------------------------------------------------------------------
// TNNTPMod2
//---------------------------------------------------------------------------

__fastcall TNNTPMod2::TNNTPMod2(TComponent* Owner)
	: TForm(Owner)
{
   //msgman = new TMsgManager(data);
   log = new TStringList();
   proxy = new TStringList();
   subjects = new TStringList();
   profile = new TStringList();
   //data = new TStringList();
   //ignoreerrors = new TStringList();
   //ignoreerrors->CommaText = ignoreConnectErrors;

   TIniFile *ini = new TIniFile(AppIni());
   nntpproxy = ini->ReadString("settings", "nntpproxy", "none");
   newsgroup = ini->ReadString("settings", "newsgroup", "alt.anonymous.messages");
   timeout = ini->ReadInteger("settings", "nntptimeout", 30) * 1000;
   maxloglines = ini->ReadInteger("settings", "loglines", 1000);
   SSLIOHandler->SSLOptions->CipherList = ini->ReadString(SSLCipherSuites, NntpSSLciphersuite, NntpSSLCipherSuite);
   delete ini;

   NNTP->Intercept = IdLogEvent;
   IdLogEvent->Active = true;

   ShowProgress(0,0,"");
}
//---------------------------------------------------------------------------

__fastcall TNNTPMod2::~TNNTPMod2()
{
   DB->SaveAccount(account, profile);
   SaveLog();

   ShowStatus("");
   if (!goterror && !timeout)
	  ShowProgress(0,0,"");

   //delete ignoreerrors;
   //delete data;
   delete profile;
   delete subjects;
   delete proxy;
   delete log;
   //delete msgman;
}
//---------------------------------------------------------------------------

char* __fastcall AStr2CStr(const AnsiString &astring)
{
   int len = astring.Length();
   char *cstr = new char[len+1];
   strncpy(cstr, astring.c_str(), len);
   cstr[len] = '\0';
   return cstr;
}
//--------------------------------------------------------------------------

void __fastcall LogTimeoutToDisk(TDateTime datetime)
{
   TIniFile *ini = new TIniFile(AppIni());
   if (ini->ReadBool("settings", "savetimeouts", false))
   {
	  TStringList *timeouts = new TStringList();
	  AnsiString timeoutsfile = AppDataDir()+"timeouts.txt";
	  if (FileExists(timeoutsfile))
		 timeouts->LoadFromFile(timeoutsfile);
	  timeouts->Add(FormatDateTime("yyyy-mm-dd hh:nn:ss", datetime));
	  timeouts->SaveToFile(timeoutsfile);
	  delete timeouts;
   }
   delete ini;
}
//--------------------------------------------------------------------------

/*bool __fastcall TNNTPMod2::ShouldIgnoreError(const AnsiString &error)
{
   bool should = false;
   for (int i = 0; i < ignoreerrors->Count; i++)
   {
	  if (error.Pos(ignoreerrors->Strings[i]) > 0)
	  {
		 should = true;
		 break;
	  }
   }
   return should;
}*/
//--------------------------------------------------------------------------

bool __fastcall TNNTPMod2::CatchUpGroup()
{
   if (ReadString(profile, "key", "").IsEmpty())
   {
	  ErrorBox("No NNTP account defined!    ");
	  return false;
   }

   server = ReadString(profile, "host", "");
   if (server.IsEmpty())
   {
	  ErrorBox("No NNTP server defined!    ");
	  return false;
   }

   if (!SetupProxy())
   {
	  ErrorBox("The proxy, "+nntpproxy+", doesn't exist!   ");
	  return false;
   }

   if (canceled)
   {
	  Display("done");
	  return false;
   }

   log->Clear();
   OwnerPerform(WM_LOGDISPLAY, CLEARLOG, 0);
   ShowStatus("preparing...");

   AnsiString datetime = FormatDateTime("yyyy-mm-dd hh:nn", Now());
   Display(datetime);
   Display("Proxy: " + GetTok(0, nntpproxy, ','));
   Display("Account: " + ReadString(profile, "key", "error"));

   if (canceled)
   {
	  Display("Done");
	  return false;
   }

   NNTP->Host = server;
   NNTP->Port = (WORD)ReadString(profile, "port", "119").ToIntDef(119);
   NNTP->Username = ReadString(profile, "userid", "");
   NNTP->Password = ReadString(profile, "password", "");
   AnsiString ssltype = ReadString(profile, "tls", "none").LowerCase();
   if (ssltype == "tls")
	  NNTP->UseTLS = utUseImplicitTLS;
   else if (ssltype == "ssl")
	  NNTP->UseTLS = utUseExplicitTLS;
   else

   NNTP->UseTLS = ssltype == "tls" ? utUseImplicitTLS : utUseExplicitTLS;

   if (canceled)
   {
	  Display("Done");
	  return false;
   }

   goterror = false;
   totalarticles = 0;
   canceled = false;
   aborted = false;
   timeout = false;
   mails = 0;

   ShowProgress(0,0,"");
   if (canceled)
   {
	  Display("Done");
	  return false;
   }

   NNTP->ConnectTimeout = timeout;
   NNTP->ReadTimeout = timeout;
   NNTP->Mode = mtReader;
   IdLogEvent->Active = true;

   if (canceled)
   {
	  CatchupComplete();
	  return false;
   }

   Display("Connecting to " + server);
   try
   {
	  NNTP->Connect();
	  if (!NNTP->Connected())
	  {
		 CatchupComplete();
		 return false;
	  }

	  try
	  {
		 NNTP->SelectGroup(newsgroup);
		 totalarticles = NNTP->MsgHigh - curmsg;
		 Display("Newsgroup: "+newsgroup+" "+AnsiString(NNTP->MsgLow)+" "+AnsiString(NNTP->MsgHigh));
		 curmsg = NNTP->MsgHigh + 1;
		 Display("New starting message: " + IntToStr(curmsg));
		 profile->Values["lastarticle"] = curmsg - 1;
		 DB->SaveAccount(account, profile);
	  }
	  catch (EIdReplyRFCError &e)
	  {
		 if (!canceled)
		 {
			Display("SelectNewsgroup "+newsgroup+" ReplyError: "+IntToStr(e.ErrorCode)+" "+e.Message);
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			goterror = true;
		 }
	  }
	  catch (EIdException &e)
	  {
		 if (!canceled)
		 {
			Display("SelectNewsgroup "+newsgroup+" IdError: "+e.Message);
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			timeout = dynamic_cast<EIdReadTimeout*>(&e) != NULL;
			goterror = true;
		 }
	  }
	  catch (Exception &e)
	  {
		 if (!canceled)
		 {
			Display("SelectNewsgroup "+newsgroup+" Error: "+e.Message);
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			goterror = true;
		 }
	  }

   }
   catch (EIdOSSLCouldNotLoadSSLLibrary &e)
   {
	  if (!canceled)
	  {
		 Display("Failed to load SSL library: "+WhichFailedToLoad());
		 OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 goterror = true;
	  }
   }
   catch (EIdNNTPConnectionRefused &e)
   {
	  if (!canceled)
	  {
		 Display("Connection refused! \nTry again later.");
		 OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 goterror = true;
	  }
   }
   catch (EIdReplyRFCError &e)
   {
	  if (!canceled)
	  {
		 Display("Connection failed! ReplyError: "+IntToStr(e.ErrorCode)+" "+e.Message);
		 //OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 //goterror = true;
	  }
   }
   catch (EIdException &e)
   {
	  AnsiString errorstr = e.Message.LowerCase();
	  if (!canceled && !errorstr.Pos("disconnected") && !errorstr.Pos("not connected"))
	  {
		 goterror = false;
		 Display("Connection failed! IdError: "+e.Message);
		 timeout = dynamic_cast<EIdConnectTimeout*>(&e) != NULL;
		 //OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 //goterror = true;
	  }
   }
   catch (Exception &e)
   {
	  AnsiString errorstr = e.Message.LowerCase();
	  if (!canceled && errorstr != "eaccessviolation")
	  {
		 Display("Connection failed! Error: "+e.Message);
		 //OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 //goterror = true;
	  }
   }

   if (NNTP->Connected())
   {
	  try
	  {
		 NNTP->Disconnect();
	  }
	  catch (EIdReplyRFCError &e)
	  {
		 AnsiString errorstr = e.Message.LowerCase();
		 if (!canceled && !errorstr.Pos("disconnected"))
		 {
			Display("Error disconnecting! ReplyError: "+IntToStr(e.ErrorCode)+" "+e.Message);
			//OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			//goterror = true;
		 }
	  }
	  catch (EIdException &e)
	  {
		 if (!canceled)
		 {
			Display("Error disconnecting! IdError: "+e.Message);
			//OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			//goterror = true;
		 }
	  }
	  catch (Exception &e)
	  {
		 if (!canceled)
		 {
			Display("Error disconnecting. Error: "+e.Message);
			//OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			//goterror = true;
		 }
	  }
   }

   try
   {
	  if (NNTP->Connected())
		 NNTP->Disconnect();
   }
   catch(...)
   {
   }

   CatchupComplete();

   return goterror;
}
//--------------------------------------------------------------------------

bool __fastcall TNNTPMod2::CheckForMail()
{
   if (ReadString(profile, "key", "").IsEmpty())
   {
	  ErrorBox("No NNTP account defined!    ");
	  return false;
   }

   server = ReadString(profile, "host", "");
   if (server.IsEmpty())
   {
	  ErrorBox("No NNTP server defined!    ");
	  return false;
   }

   if (!SetupProxy())
   {
	  ErrorBox("The proxy, "+nntpproxy+", doesn't exist!   ");
	  return false;
   }

   if (canceled)
   {
	  Display("done");
	  return false;
   }

   log->Clear();
   OwnerPerform(WM_LOGDISPLAY, CLEARLOG, 0);
   ShowStatus("Preparing...");

   AnsiString datetime = FormatDateTime("yyyy-mm-dd hh:nn", Now());
   Display(datetime);
   Display("Proxy: " + GetTok(0, nntpproxy, ','));
   Display("Account: " + ReadString(profile, "key", "error"));

   if (canceled)
   {
	  Display("Done");
	  return false;
   }

   NNTP->Host = server;
   NNTP->Port = (WORD)ReadString(profile, "port", "119").ToIntDef(119);
   NNTP->Username = ReadString(profile, "userid", "");
   NNTP->Password = ReadString(profile, "password", "");
   AnsiString ssltype = ReadString(profile, "tls", "none").LowerCase();
   if (ssltype == "tls")
	  NNTP->UseTLS = utUseImplicitTLS;
   else if (ssltype == "ssl")
	  NNTP->UseTLS = utUseExplicitTLS;
   else

   NNTP->UseTLS = ssltype == "tls" ? utUseImplicitTLS : utUseExplicitTLS;

   curmsg = ReadInteger(profile, "lastarticle", -1);
   if (curmsg > 0)
	  curmsg++;

   if (canceled)
   {
	  Display("Done");
	  return false;
   }

   goterror = false;
   displaying = true;
   totalarticles = 0;
   canceled = false;
   aborted = false;
   timeout = false;
   mails = 0;

   // get subjects to compare -- eliminate inactive subjects
   DB->LoadSubjects(subjects);
   int x = 0; while (x < subjects->Count)
   {
	  if (!subjects->Strings[x].Pos("active=1"))
		 subjects->Delete(x);
	  else
		 x++;
   }

   //connecting = true;
   ShowProgress(0,0,"");
   //OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTART, 0);
   if (canceled)
   {
	  Display("Done");
	  return false;
   }
   TStringList *data = new TStringList();

   NNTP->ConnectTimeout = timeout;
   NNTP->ReadTimeout = timeout;
   NNTP->Mode = mtReader;
   IdLogEvent->Active = true;

   if (canceled)
   {
	  CheckComplete();
	  delete data;
	  return false;
   }

   Display("Connecting to " + server);
   try
   {
	  NNTP->Connect();
	  if (!NNTP->Connected())
	  {
		 CheckComplete();
		 delete data;
		 return false;
	  }

	  try
	  {
		 NNTP->SelectGroup(newsgroup);
		 if (curmsg < 0)
		 {
			__int64 howmany = HowManyMessages((TForm*)Owner, NNTP->MsgHigh - NNTP->MsgLow);
			if (howmany < 0)
			{
			   CheckComplete();
			   delete data;
			   return false;
			}
			curmsg = NNTP->MsgHigh - howmany;
		 }
		 if (curmsg < NNTP->MsgLow)
			curmsg = NNTP->MsgLow;

		 Display("Newsgroup: "+newsgroup+" "+AnsiString(NNTP->MsgLow)+" "+AnsiString(NNTP->MsgHigh));
		 Display("Starting message: "+AnsiString(curmsg));
		 ShowStatus("Processing subjects...");

		 try
		 {
			NNTP->XHDR("subject", AnsiString(curmsg)+"-", data);
			for (int i = 0; i < data->Count; i++)
			{
			   try
			   {                             // subject
				  if (SubjectMatch(GetTok(1, data->Strings[i], ' ')))
				  {
					 mails++;
				  }
			   }
			   catch(EConvertError &e)
			   {
					 // ignore converterror = not a hashed or esubed subject
			   }
			}
		 }
		 catch (EIdReplyRFCError &e)
		 {
		 }
		 catch (EIdException &e)
		 {
		 }
	  }
	  catch (EIdReplyRFCError &e)
	  {
		 if (!canceled)
		 {
			Display("SelectNewsgroup "+newsgroup+" ReplyError: "+IntToStr(e.ErrorCode)+" "+e.Message);
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			goterror = true;
		 }
	  }
	  catch (EIdException &e)
	  {
		 if (!canceled)
		 {
			Display("SelectNewsgroup "+newsgroup+" IdError: "+e.Message);
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			timeout = dynamic_cast<EIdReadTimeout*>(&e) != NULL;
			goterror = true;
		 }
	  }
	  catch (Exception &e)
	  {
		 if (!canceled)
		 {
			Display("SelectNewsgroup "+newsgroup+" Error: "+e.Message);
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			goterror = true;
		 }
	  }

   }
   catch (EIdOSSLCouldNotLoadSSLLibrary &e)
   {
	  if (!canceled)
	  {
		 Display("Failed to load SSL library: "+WhichFailedToLoad());
		 OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 goterror = true;
	  }
   }
   catch (EIdNNTPConnectionRefused &e)
   {
	  if (!canceled)
	  {
		 Display("Connection refused! \nTry again later.");
		 OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 goterror = true;
	  }
   }
   catch (EIdReplyRFCError &e)
   {
	  if (!canceled)
	  {
		 Display("Connection failed! ReplyError: "+IntToStr(e.ErrorCode)+" "+e.Message);
		 //OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 //goterror = true;
	  }
   }
   catch (EIdException &e)
   {
	  AnsiString errorstr = e.Message.LowerCase();
	  if (!canceled && !errorstr.Pos("disconnected") && !errorstr.Pos("not connected"))
	  {
		 Display("Connection failed! IdError: "+e.Message);
		 timeout = dynamic_cast<EIdConnectTimeout*>(&e) != NULL;
		 //OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 //goterror = true;
	  }
   }
   catch (Exception &e)
   {
	  AnsiString errorstr = e.Message.LowerCase();
	  if (!canceled && errorstr != "eaccessviolation")
	  {
		 Display("Connection failed! Error: "+e.Message);
		 //OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 //goterror = true;
	  }
   }

   if (NNTP->Connected())
   {
	  try
	  {
		 NNTP->Disconnect();
	  }
	  catch (EIdReplyRFCError &e)
	  {
		 AnsiString errorstr = e.Message.LowerCase();
		 if (!canceled && !errorstr.Pos("disconnected"))
		 {
			Display("Error disconnecting! ReplyError: "+IntToStr(e.ErrorCode)+" "+e.Message);
			//OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			//goterror = true;
		 }
	  }
	  catch (EIdException &e)
	  {
		 if (!canceled)
		 {
			Display("Error disconnecting! IdError: "+e.Message);
			//OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			//goterror = true;
		 }
	  }
	  catch (Exception &e)
	  {
		 if (!canceled)
		 {
			Display("Error disconnecting. Error: "+e.Message);
			//OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			//goterror = true;
		 }
	  }
   }

   try
   {
	  if (NNTP->Connected())
		 NNTP->Disconnect();
   }
   catch(...)
   {
   }

   CheckComplete();

   delete data;
   return goterror;
}
//--------------------------------------------------------------------------

bool __fastcall TNNTPMod2::GetTheMail()
{
   if (ReadString(profile, "key", "").IsEmpty())
   {
	  ErrorBox("No NNTP account defined!    ");
	  return false;
   }

   server = ReadString(profile, "host", "");
   if (server.IsEmpty())
   {
	  ErrorBox("No NNTP server defined!    ");
	  return false;
   }

   if (!SetupProxy())
   {
	  ErrorBox("The proxy, "+nntpproxy+", doesn't exist!   ");
	  return false;
   }

   if (canceled)
   {
	  Display("done");
	  return false;
   }

   log->Clear();
   OwnerPerform(WM_LOGDISPLAY, CLEARLOG, 0);
   ShowStatus("preparing...");

   AnsiString datetime = FormatDateTime("yyyy-mm-dd hh:nn", Now());
   Display(datetime);
   Display("Proxy: " + GetTok(0, nntpproxy, ','));
   Display("Account: " + ReadString(profile, "key", "error"));

   if (canceled)
   {
	  Display("Done");
	  return false;
   }

   NNTP->Host = server;
   NNTP->Port = (WORD)ReadString(profile, "port", "119").ToIntDef(119);
   NNTP->Username = ReadString(profile, "userid", "");
   NNTP->Password = ReadString(profile, "password", "");
   AnsiString ssltype = ReadString(profile, "tls", "none").LowerCase();
   if (ssltype == "tls")
	  NNTP->UseTLS = utUseImplicitTLS;
   else if (ssltype == "ssl")
	  NNTP->UseTLS = utUseExplicitTLS;
   else

   NNTP->UseTLS = ssltype == "tls" ? utUseImplicitTLS : utUseExplicitTLS;

   curmsg = ReadInteger(profile, "lastarticle", -1);
   if (curmsg > 0)
	  curmsg++;

   if (canceled)
   {
	  Display("Done");
	  return false;
   }

   goterror = false;
   displaying = true;
   totalarticles = 0;
   canceled = false;
   aborted = false;
   timeout = false;
   mails = 0;

   // get subjects to compare -- eliminate inactive subjects
   DB->LoadSubjects(subjects);
   int x = 0; while (x < subjects->Count)
   {
	  if (!subjects->Strings[x].Pos("active=1"))
		 subjects->Delete(x);
	  else
		 x++;
   }

   //connecting = true;
   ShowProgress(0,0,"");
   //OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTART, 0);
   if (canceled)
   {
	  Display("Done");
	  return false;
   }
   TStringList *msg = new TStringList();
   TStringList *data = new TStringList();
   TMsgManager *msgman = new TMsgManager(msg);

   NNTP->ConnectTimeout = timeout;
   NNTP->ReadTimeout = timeout;
   NNTP->Mode = mtReader;
   IdLogEvent->Active = true;

   if (canceled)
   {
	  RetrieveComplete();
	  delete msgman;
	  delete data;
	  delete msg;
	  return false;
   }

   Display("Connecting to " + server);
   try
   {
	  NNTP->Connect();
	  if (!NNTP->Connected())
	  {
		 RetrieveComplete();
		 delete msgman;
		 delete data;
		 delete msg;
		 return false;
	  }

	  try
	  {
		 NNTP->SelectGroup(newsgroup);
		 if (curmsg < 0)
		 {
			__int64 howmany = HowManyMessages((TForm*)Owner, NNTP->MsgHigh - NNTP->MsgLow);
			if (howmany < 0)
			{
			   RetrieveComplete();
			   delete msgman;
			   delete data;
			   delete msg;
			   return false;
			}
			curmsg = NNTP->MsgHigh - howmany;
		 }
		 if (curmsg < NNTP->MsgLow)
			curmsg = NNTP->MsgLow;

		 totalarticles = NNTP->MsgHigh - curmsg;
		 Display("Newsgroup: "+newsgroup+" "+AnsiString(NNTP->MsgLow)+" "+AnsiString(NNTP->MsgHigh));
		 Display("Starting message: "+AnsiString(curmsg));
		 next = 0;

		 while (curmsg <= NNTP->MsgHigh)
		 {
			if (canceled)
			   break;

			try
			{
			   if (!NNTP->Connected())
				  break;

			   NNTP->GetArticle(curmsg, msg);
			   if (NNTP->LastCmdResult->NumericCode == 220)
			   {
				  try
				  {
					 if (SubjectMatch(msgman))
					 {
						msg->Insert(0, curmsg);
						OwnerPerform(WM_SAVEMESSAGE, ++mails, (LPARAM)AStr2CStr(msg->Text));
					 }
				  }
				  catch(EConvertError &e)
				  {
					 // ignore converterror = not a hashed or esubed subject
				  }
			   }
			}
			catch (EIdReplyRFCError &e)
			{
			}
			catch (EIdException &e)
			{
			}

			ShowProgress(next, totalarticles, curmsg);
			msg->Clear();
			curmsg++;
			next++;
		 }
	  }
	  catch (EIdReplyRFCError &e)
	  {
		 if (!canceled)
		 {
			Display("SelectNewsgroup "+newsgroup+" ReplyError: "+IntToStr(e.ErrorCode)+" "+e.Message);
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			goterror = true;
		 }
	  }
	  catch (EIdException &e)
	  {
		 if (!canceled)
		 {
			Display("SelectNewsgroup "+newsgroup+" IdError: "+e.Message);
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			goterror = true;
			timeout = dynamic_cast<EIdReadTimeout*>(&e) != NULL;
		 }
	  }
	  catch (Exception &e)
	  {
		 if (!canceled)
		 {
			Display("SelectNewsgroup "+newsgroup+" Error: "+e.Message);
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			goterror = true;
		 }
	  }

   }
   catch (EIdOSSLCouldNotLoadSSLLibrary &e)
   {
	  if (!canceled)
	  {
		 Display("Failed to load SSL library: "+WhichFailedToLoad());
		 OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 goterror = true;
	  }
   }
   catch (EIdNNTPConnectionRefused &e)
   {
	  if (!canceled)
	  {
		 Display("Connection refused! \nTry again later.");
		 OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 goterror = true;
	  }
   }
   catch (EIdReplyRFCError &e)
   {
	  if (!canceled)
	  {
		 Display("Connection failed! ReplyError: "+IntToStr(e.ErrorCode)+" "+e.Message);
		 //OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 //goterror = true;
	  }
   }
   catch (EIdException &e)
   {
	  AnsiString errorstr = e.Message.LowerCase();
	  if (!canceled && !errorstr.Pos("disconnected") && !errorstr.Pos("not connected"))
	  {
		 Display("Connection failed! IdError: "+e.Message);
		 timeout = dynamic_cast<EIdConnectTimeout*>(&e) != NULL;
		 if (errorstr.Pos("connection refused"))
		 {
			OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			goterror = true;
		 }
	  }
   }
   catch (Exception &e)
   {
	  AnsiString errorstr = e.Message.LowerCase();
	  if (!canceled && errorstr != "eaccessviolation")
	  {
		 Display("Connection failed! Error: "+e.Message);
		 //OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
		 //goterror = true;
	  }
   }

   if (NNTP->Connected())
   {
	  try
	  {
		 NNTP->Disconnect();
	  }
	  catch (EIdReplyRFCError &e)
	  {
		 AnsiString errorstr = e.Message.LowerCase();
		 if (!canceled && !errorstr.Pos("disconnected"))
		 {
			Display("Error disconnecting! ReplyError: "+IntToStr(e.ErrorCode)+" "+e.Message);
			//OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			//goterror = true;
		 }
	  }
	  catch (EIdException &e)
	  {
		 if (!canceled)
		 {
			Display("Error disconnecting! IdError: "+e.Message);
			//OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			//goterror = true;
		 }
	  }
	  catch (Exception &e)
	  {
		 if (!canceled)
		 {
			Display("Error disconnecting. Error: "+e.Message);
			//OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(log->Text));
			//goterror = true;
		 }
	  }
   }

   try
   {
	  if (NNTP->Connected())
		 NNTP->Disconnect();
   }
   catch(...)
   {
   }
   
   RetrieveComplete();

   delete msgman;
   delete data;
   delete msg;
   return goterror;
}
//-------------------------------------------------------------------------

void __fastcall TNNTPMod2::RetrieveComplete()
{
   if (curmsg < 0)
	  curmsg = 1;

   if (!timeout)
   {
	  Display("Ending message: " + IntToStr(curmsg-1));
	  profile->Values["lastarticle"] = curmsg-1;
	  DB->SaveAccount(account, profile);
   }

   if (timeout)
   {
	  Display("Connection closed!");
	  Display("Failed!");
	  LogTimeoutToDisk(Now());
   }
   else
   {
	  if (mails)
		 Display(IntToStr(mails)+AnsiString(" new message")+(mails>1?"s":""));
	  else
		 Display("No mail");
   }

   if (!goterror)
	  ShowProgress(0,0,"");
   OwnerPerform(WM_UPDATECOMPLETE, canceled, mails); //catchup ? 0 : mails);
   if (mails)
	  OwnerPerform(WM_SAVEDATABASE, 0, 0);
   OwnerPerform(WM_DELNNTPTHREAD, 0, 0);
}
//-------------------------------------------------------------------------

void __fastcall TNNTPMod2::CheckComplete()
{
   if (curmsg < 0)
	  curmsg = 1;

   if (!timeout)
   {
	  Display("Ending message: " + IntToStr(curmsg-1));
   }

   if (timeout)
   {
	  Display("Connection closed!");
	  Display("Failed!");
	  LogTimeoutToDisk(Now());
   }
   else
   {
	  if (mails)
		 Display(IntToStr(mails)+AnsiString(" waiting message")+(mails>1?"s":""));
	  else
		 Display("No mail");
   }

   OwnerPerform(WM_UPDATECOMPLETE, canceled, mails);
   if (!goterror)
	  ShowProgress(0,0,"");
   OwnerPerform(WM_DELNNTPTHREAD, 0, 0);
}
//-------------------------------------------------------------------------

void __fastcall TNNTPMod2::CatchupComplete()
{
   if (timeout)
   {
	  Display("Connection closed!");
	  Display("Failed!");
   }
   else
   {
	  Display("Newsgroup caught up!");
	  Display("Done");
   }

   if (!goterror)
	  ShowProgress(0,0,"");
   OwnerPerform(WM_UPDATECOMPLETE, canceled, 0);
   OwnerPerform(WM_DELNNTPTHREAD, 0, 0);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::WMNNTPTimeout(TMessage &Msg)
{
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::WMNNTPCancel(TMessage &Msg)
{
   canceled = true;
   Display("User canceled operation!");
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::SaveLog()
{
   TStringList *lines = new TStringList();
   DB->GetText("log", lines);
   lines->Add("");
   lines->AddStrings(log);
   while (lines->Count > maxloglines)
	  lines->Delete(0);
   DB->PutText("log", lines);
   delete lines;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::ShowProgress(const double current, const double total, const AnsiString &article)
{
   AnsiString text;
   if (current && total)
   {
	  text = IntToStr((int)current)+" of "+IntToStr((int)total);
	  if (mails)
		 text += "  ("+IntToStr(mails)+")";
   }
   WORD Pcnt = (WORD)PercentComplete(current);
   OwnerPerform(WM_SHOWPROGRESS, (WORD)Pcnt, (LPARAM)AStr2CStr(text));
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::ShowStatus(const AnsiString &msg)
{
   OwnerPerform(WM_SHOWSTATUS, 0 , (LPARAM)AStr2CStr(msg));
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::Display(const AnsiString &msg)
{
   //displayMsg = msg;
   OwnerPerform(WM_LOGDISPLAY, 0, (LPARAM)AStr2CStr(msg));
   log->Add(msg);
   if (msg.Pos("Connect") == 1)
	  ShowStatus(msg);
   else
	  ShowStatus("");
   Snooze(10);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::OwnerPerform(UINT msg, WPARAM wparam, LPARAM lparam)
{
   PostMessage(MainFormHandle, msg, wparam, lparam);
   Snooze(100);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::ErrorBox(const AnsiString &Text)
{
   AnsiString data = Text +"|" + IntToStr(MT_ERROR) + "|" + IntToStr(MB_OK);
   OwnerPerform(WM_MESSAGEBOX, 0, (LPARAM)data.c_str());
}
//---------------------------------------------------------------------------

bool __fastcall TNNTPMod2::SetupProxy()
{
   SocksInfo->Host = "";
   SocksInfo->Port = 119;
   SocksInfo->Version = svNoSocks;
   SocksInfo->Authentication = saNoAuthentication;
   SocksInfo->Username = "";
   SocksInfo->Password  = "";
   assert(nntpproxy != "");

   if (nntpproxy != "none")
   {
	  TStringList *proxies = new TStringList();
	  DB->GetText("proxies", proxies);
	  proxy->CommaText = proxies->Values[nntpproxy];
	  delete proxies;
	  if (proxy->Text.IsEmpty())
		 return false;

	  assert(proxy->Count > 0);
	  assert(proxy->Values["host"] != "");
	  assert(proxy->Values["port"] != "");
	  assert(proxy->Values["socks"] != "" && proxy->Values["socks"] != "no");
	  SocksInfo->Host = proxy->Values["host"];
	  SocksInfo->Port = (WORD)proxy->Values["port"].ToIntDef(119);
	  AnsiString level = proxy->Values["socks"].LowerCase();
	  if (level == "4") SocksInfo->Version = svSocks4;
	  else if (level == "4a") SocksInfo->Version = svSocks4A;
	  else if (level == "5") SocksInfo->Version = svSocks5;

	  if (ReadBool(proxy, "auth", false))
	  {
		 assert(proxy->Values["userid"] != "");
		 assert(proxy->Values["password"] != "");
		 SocksInfo->Authentication = saUsernamePassword;
		 SocksInfo->Password = proxy->Values["password"];
		 SocksInfo->Username = proxy->Values["userid"];
	  }
   }
   return true;
}
//---------------------------------------------------------------------------

int __fastcall TNNTPMod2::PercentComplete(double next)
{
   if (totalarticles > 0)
	  return (int)((next / totalarticles) * 100);
   return (int)next;
}
//---------------------------------------------------------------------------

bool __fastcall TNNTPMod2::SubjectMatch(const AnsiString &msgsubject)
{
   bool ok;
   AnsiString line, ephrase, cmpsubject;
   assert(msgsubject != "");

   TStringList *subdata = new TStringList();
   for (int i = 0; i < subjects->Count; i++)
   {
	  subdata->CommaText = subjects->Strings[i],
	  assert(subdata->Count > 0);
	  if (subdata->Values["active"] != "1")
		 continue;
	  cmpsubject = subdata->Values["subject"];
	  assert(cmpsubject != "");

	  if (subdata->Values["esub"] == "1")
	  {
		 ephrase = subdata->Values["ephrase"];
		 assert(ephrase != "");
		 ok = IsSubjectBLOWeSub(msgsubject, cmpsubject, ephrase);
		 if (!ok)
			ok = IsSubjectIDEAeSub(msgsubject, cmpsubject, ephrase);
		 if (ok)
			break;
	  }
	  else
	  {
		 ok = TestSubjectHSUB(msgsubject, cmpsubject);
		 if (ok)
			break;
	  }
   }

   delete subdata;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TNNTPMod2::SubjectMatch(TMsgManager *msg)
{
   AnsiString cmpsubject, ephrase, kphrase, subno,
	  msgsubject = msg->GetHeader("Subject:");
   assert(msgsubject != "");

   bool ok;
   TStringList *subdata = new TStringList();
   for (int i = 0; i < subjects->Count; i++)
   {
	  subdata->CommaText = subjects->Strings[i],
	  assert(subdata->Count > 0);
	  if (subdata->Values["active"] != "1")
		 continue;

	  cmpsubject = subdata->Values["subject"];
	  assert(cmpsubject != "");

	  if (subdata->Values["esub"] == "1")
	  {
		 ephrase = subdata->Values["ephrase"];
		 assert(ephrase != "");
		 ok = IsSubjectBLOWeSub(msgsubject, cmpsubject, ephrase);
		 if (!ok)
			ok = IsSubjectIDEAeSub(msgsubject, cmpsubject, ephrase);
		 if (ok)
		 {
			msg->Lines->Insert(0, "Subject-No: "+subdata->Values["subno"]);
			break;
		 }
	  }
	  else
	  {
		 ok = TestSubjectHSUB(msgsubject, cmpsubject);
		 if (ok)
		 {
			msg->Lines->Insert(0, "Subject-No: "+subdata->Values["subno"]);
			break;
		 }
	  }
   }
   delete subdata;
   return ok;
}

//---------------------------------------------------------------------------
// IdNNTP Client
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPDisconnected(TObject *Sender)
{
//look(log);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPNewNewsList(AnsiString AMsgID,
	  bool &ACanContinue)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPStatus(TObject *ASender, const TIdStatus AStatus,
	  const AnsiString AStatusText)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPTLSNotAvailable(TObject *Asender,
	  bool &VContinue)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPWorkEnd(TObject *ASender, TWorkMode AWorkMode)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPXHDREntry(AnsiString AHeader, AnsiString AMsg,
	  AnsiString AHeaderData, bool &ACanContinue)
{
//
}
//---------------------------------------------------------------------------


void __fastcall TNNTPMod2::NNTPNewGroupsList(AnsiString ANewsgroup,
	  __int64 ALow, __int64 AHigh, AnsiString AType, bool &ACanContinue)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPNewsgroupList(AnsiString ANewsgroup,
	  __int64 ALow, __int64 AHigh, AnsiString AType, bool &ACanContinue)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPWork(TObject *ASender, TWorkMode AWorkMode,
	  __int64 AWorkCount)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPWorkBegin(TObject *ASender, TWorkMode AWorkMode,
	  __int64 AWorkCountMax)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPXOVER(__int64 AArticleIndex, AnsiString ASubject,
	  AnsiString AFrom, TDateTime ADate, AnsiString AMsgId,
	  AnsiString AReferences, int AByteCount, int ALineCount,
	  AnsiString AExtraData, bool &VCanContinue)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::NNTPConnected(TObject *Sender)
{
//
}
//---------------------------------------------------------------------------

AnsiString __fastcall striptrailingCrLf(AnsiString line)
{
   int p;
   while ((p = line.Pos("\r")) > 0)
	  line.Delete(p, 1);
   while ((p = line.Pos("\n")) > 0)
	  line.Delete(p, 1);
   return line;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::IdLogEventReceived(TComponent *ASender,
	  const AnsiString AText, const AnsiString AData)
{
   AnsiString line = striptrailingCrLf(AData);
   if (displaying)
   {
	  int code = GetTok(0, line, ' ').ToIntDef(-1);
	  if (code > 199)
	  {
		 if (!(code == 200 && log->Count == 5))
		    log->Add(line);
	  }
	  if (line.Pos("> article") > 0)
	  {
		 displaying = false;
	  }
   }
   else
   {
	  displaying = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::IdLogEventSent(TComponent *ASender,
	  const AnsiString AText, const AnsiString AData)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::IdLogEventReceive(TIdConnectionIntercept *ASender,
	  TIdBytes &ABuffer)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::IdLogEventSend(TIdConnectionIntercept *ASender,
	  TIdBytes &ABuffer)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::IdLogEventStatus(TComponent *ASender,
	  const AnsiString AText)
{
   log->Add(AText);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::IdLogEventConnect(TIdConnectionIntercept *ASender)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod2::IdLogEventDisconnect(TIdConnectionIntercept *ASender)
{
//
}
//---------------------------------------------------------------------------


