/*
*  dbs.cpp
*
*  Copyright (C) 2012 Richard Christman
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver AAM, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver AAM grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "dbs.h"
#include "assert.h"
#include "zlib.h"
#include "zconf.h"
#include "utility.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//TDB *DB;
//---------------------------------------------------------------------------
// TDB
//---------------------------------------------------------------------------

TDB* __fastcall OpenDB(const AnsiString &FileName, bool InMemory)
{
   assert(FileName != "");
   TDB *DB = new TDB(FileName, InMemory);
   if (DB->error)
   {
      delete DB;
      return NULL;
   }
   return DB;
}
//---------------------------------------------------------------------------

TDB* __fastcall OpenDB(const AnsiString &FileName, TStream *Stream)
{
   assert(FileName != "");
   TDB *DB = new TDB(FileName, Stream);
   if (DB->error)
   {
      delete DB;
      return NULL;
   }
   return DB;
}
//---------------------------------------------------------------------------

bool __fastcall CloseDB(TDB *DB)
{
   delete DB;
   return true;
}

//---------------------------------------------------------------------------
// TDB
//---------------------------------------------------------------------------

__fastcall TDB::TDB(const AnsiString &FileName, bool InMemory)
   : TObject()
{
   error == DB_OK;
   ffilename = FileName;
   inmemory = InMemory;
   OpenDatabase();
}
//---------------------------------------------------------------------------

__fastcall TDB::TDB(const AnsiString &FileName, TStream *Stream)
   : TObject()
{
   fstream = Stream;
   error == DB_OK;
   ffilename = FileName;
   inmemory = dynamic_cast<TMemoryStream*>(Stream) != NULL;
   OpenDatabase();
}
//---------------------------------------------------------------------------

__fastcall TDB::~TDB()
{
   CloseDatabase();
   delete index[0];
   delete dbman;
   // its not modified. saving is now done in closequery event
   if (modified)
	  SaveToFile(filename);

   if (delstream)
	  delete stream;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::OpenDatabase()
{
   if (!stream)
   {
      bool exists = FileExists(filename);
	  try
	  {
         if (inmemory)
         {
            fstream = new TMemoryStream();
            if (exists)
               ((TMemoryStream*)stream)->LoadFromFile(filename);
         }
         else
		 {
            fstream = new TFileStream(filename,
               (unsigned short)(exists ? fmOpenReadWrite : fmCreate));
         }
      }
      catch(Exception &e)
      {
         exceptmsg = e.Message;
         error = DB_READEXC;
         ShowDBError(error);
         return false;
      }
   }

   dbman = new TDBmanager(this, stream, filename);
   if (!dbman->error)
   {
	  index[0] = new TDBindex(dbman);
      modified = false;
   }
   else
   {
      ShowDBError(dbman->error);
      error = dbman->error;
   }

   delstream = true;
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::CloseDatabase()
{
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TDB::SetFileName(const AnsiString &Filename)
{
   ffilename = Filename;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TDB::GetFileName()
{
   return ffilename;
}
//---------------------------------------------------------------------------

TStream* __fastcall TDB::GetStream()
{
   return fstream;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::GetText(const AnsiString &path, TStrings *text)
{
   TDBbuffer *buf = new TDBbuffer();
   if (GetBuffer(path, buf))
      text->SetText(buf->data);
   delete buf;

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::PutText(const AnsiString &path, TStrings *text)
{
   TDBbuffer *buf = NULL;
   if (text)
      buf = new TDBbuffer(text->Text.c_str());
   PutBuffer(path, buf);
   if (buf)
      delete buf;
   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::PutBuffer(const AnsiString &path, void *buf, int length,
   bool saveindex)
{
   TDBbuffer *buffer = new TDBbuffer((char*)buf, length);
   bool ok = PutBuffer(path, buffer, saveindex);
   delete buffer;
   return ok;
}
//---------------------------------------------------------------------------

/*bool __fastcall TDB::GetStream(const AnsiString &path, TMemoryStream *stream)
{
   stream->SetSize(0);
   TDBbuffer *buffer = new TDBbuffer();
   bool ok = GetBuffer(path, buffer);
   if (ok)
	   stream->Write(buffer->data, buffer->length);
   delete buffer;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::PutStream(const AnsiString &path, TMemoryStream *stream)
{
   TDBbuffer *buffer = new TDBbuffer(stream->Memory, (int)stream->Size);
   bool ok = PutBuffer(path, buffer);
   delete buffer;
   return ok;
}*/
//---------------------------------------------------------------------------

bool __fastcall TDB::PutBuffer(const AnsiString &path, TDBbuffer *buffer,
   bool saveindex)
{
   dbman->error = DB_OK;

   try
   {
	  AnsiString nodetext;
	  int i = 1;
	  while ((nodetext = NextStep(i - 1, path)) != NULL)
	  {
		 if (NextStep(i, path).IsEmpty())
		 {
			index[i - 1]->PutBinary(nodetext, buffer, saveindex);
			break;
		 }

		 index[i] = new TDBindex(dbman, nodetext, index[i-1]);
		 if (dbman->error)
			break;

		 i++;
	  }

	  for (i = i - 1; i > 1; i--)
		 delete index[i], index[i] = NULL;
      modified = true;
   }
   __finally
   {
      if (dbman->error)
      {
         ShowDBError(dbman->error);
      }
   }

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::GetBuffer(const AnsiString &path, TDBbuffer *buffer)
{
   dbman->error = DB_OK;

   AnsiString nodetext;
   try
   {
	  int i = 1;
	  while ((nodetext = NextStep(i - 1, path)) != NULL)
	  {
		 if (NextStep(i, path).IsEmpty())
		 {
			if (index[i - 1]->GetSize(nodetext) > 0)
				index[i - 1]->GetBinary(nodetext, buffer);
			break;
		 }

		 index[i] = new TDBindex(dbman, nodetext, index[i - 1]);
		 if (dbman->error)
			break;

		 i++;
	  }

	  for (i = i - 1; i > 1; i--)
		 delete index[i], index[i] = NULL;
   }
   __finally
   {
	  if (dbman->error)
	  {
		 ShowDBError(dbman->error);
	  }
   }

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::DataExists(const AnsiString &path)
{
   int ofs;
   bool ok = GetDataOffset(path, ofs);
   return ok && ofs > 0;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::DelDataKey(const AnsiString &path)  // delete key but not data
{
   dbman->error = DB_OK;

   try
   {
      int i = 1;
      AnsiString nodetext;
      while ((nodetext = NextStep(i - 1, path)) != NULL)
	  {
         if (NextStep(i, path).IsEmpty())
         {
            index[i - 1]->DelKey(nodetext);
            break;
         }

         index[i] = new TDBindex(dbman, nodetext, index[i-1]);
         if (dbman->error)
            break;

		 i++;
      }

      for (i = i - 1; i > 1; i--)
         delete index[i], index[i] = NULL;
      modified = true;
   }
   __finally
   {
      if (dbman->error)
	  {
         ShowDBError(dbman->error);
      }
   }

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::DelItem(const AnsiString &path)
{
   bool ok = PutBuffer(path, NULL);
   return ok;
}
//---------------------------------------------------------------------------
// renitem and movedata (next down) do the same thing

bool __fastcall TDB::RenItem(const AnsiString &curname, const AnsiString &newname)
{
   dbman->error = DB_OK;

   int offset;
   if (GetDataOffset(curname, offset) && PutDataOffset(newname, offset))
      DelDataKey(curname);

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::MoveData(const AnsiString &Path, const AnsiString &NewPath)
{
   dbman->error = DB_OK;

   int offset;
   if (GetDataOffset(Path, offset) && PutDataOffset(NewPath, offset))
      DelDataKey(Path);

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::PutDataOffset(const AnsiString &path, int offset)
{
   if (dbman->error)
      return false;

   try
   {
      int i = 1;
      AnsiString nodetext;
      while ((nodetext = NextStep(i - 1, path)) != NULL)
      {
         if (NextStep(i, path).IsEmpty())
		 {
            index[i - 1]->PutOffset(nodetext, offset);
            break;
         }

         index[i] = new TDBindex(dbman, nodetext, index[i-1]);
         if (dbman->error)
            break;

         i++;
      }

      for (i = i - 1; i > 1; i--)
         delete index[i], index[i] = NULL;
	  modified = true;
   }
   __finally
   {
      if (dbman->error)
      {
         ShowDBError(dbman->error);
      }
   }

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::GetDataOffset(const AnsiString &path, int &offset)
{
   offset = 0;

   if (dbman->error)
      return false;

   try
   {
	  int i = 1;
      AnsiString nodetext;
      while ((nodetext = NextStep(i - 1, path)) != NULL)
      {
         if (NextStep(i, path).IsEmpty())
         {
            offset = index[i - 1]->GetOffset(nodetext);
            break;
         }

         index[i] = new TDBindex(dbman, nodetext, index[i - 1]);
         if (dbman->error)
			break;

         i++;
      }

	  for (i = i - 1; i > 1; i--)
         delete index[i], index[i] = NULL;
   }
   __finally
   {
      if (dbman->error)
      {
         ShowDBError(dbman->error);
      }
   }

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::SwapKey(const AnsiString &path, const AnsiString &newkey)
{
   dbman->error = DB_OK;

   try
   {
      int i = 1;
      AnsiString nodetext;
      while ((nodetext = NextStep(i - 1, path)) != NULL)
      {
         if (NextStep(i, path).IsEmpty())
         {
            index[i - 1]->SwapKey(nodetext, newkey);
            break;
         }

         index[i] = new TDBindex(dbman, nodetext, index[i-1]);
         if (dbman->error)
            break;

         i++;
      }

      for (i = i - 1; i > 1; i--)
         delete index[i], index[i] = NULL;
      modified = true;
   }
   __finally
   {
      if (dbman->error)
      {
         ShowDBError(dbman->error);
      }
   }

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::FlagKey(const AnsiString &path, int flagsbyte, byte flag)
{
   dbman->error = DB_OK;

   try
   {
	  int i = 1;
	  AnsiString nodetext;
	  while ((nodetext = NextStep(i - 1, path)) != NULL)
	  {
		 if (NextStep(i, path).IsEmpty())
		 {
			index[i - 1]->FlagByte(nodetext, flagsbyte, flag);
			break;
		 }

		 index[i] = new TDBindex(dbman, nodetext, index[i-1]);
		 if (dbman->error)
			break;

		 i++;
	  }

	  for (i = i - 1; i > 1; i--)
		 delete index[i], index[i] = NULL;
	  modified = true;
   }
   __finally
   {
	  if (dbman->error)
	  {
		 ShowDBError(dbman->error);
	  }
   }

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::UnFlagKey(const AnsiString &path, int flagsbyte, byte flag)
{
   dbman->error = DB_OK;

   try
   {
	  int i = 1;
	  AnsiString nodetext;
	  while ((nodetext = NextStep(i - 1, path)) != NULL)
	  {
		 if (NextStep(i, path).IsEmpty())
		 {
			index[i - 1]->UnFlagByte(nodetext, flagsbyte, flag);
			break;
		 }

		 index[i] = new TDBindex(dbman, nodetext, index[i-1]);
		 if (dbman->error)
			break;

		 i++;
	  }

	  for (i = i - 1; i > 1; i--)
		 delete index[i], index[i] = NULL;
	  modified = true;
   }
   __finally
   {
	  if (dbman->error)
	  {
		 ShowDBError(dbman->error);
	  }
   }

   return !dbman->error;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::KeyIsFlagged(const AnsiString &path, int flagsbyte, byte flag)
{
   dbman->error = DB_OK;
   bool flagged = false;
   try
   {
	  int i = 1;
	  AnsiString nodetext;
	  while ((nodetext = NextStep(i - 1, path)) != NULL)
	  {
		 if (NextStep(i, path).IsEmpty())
		 {
			flagged = index[i - 1]->ByteIsFlagged(nodetext, flagsbyte, flag);
			break;
		 }

		 index[i] = new TDBindex(dbman, nodetext, index[i-1]);
		 if (dbman->error)
			break;

		 i++;
	  }

	  for (i = i - 1; i > 1; i--)
		 delete index[i], index[i] = NULL;
	  modified = true;
   }
   __finally
   {
	  if (dbman->error)
	  {
		 ShowDBError(dbman->error);
	  }
   }

   return flagged;
}
//---------------------------------------------------------------------------

bool __fastcall TDB::SaveToFile(const AnsiString &FileName)
{
   if (FileName.IsEmpty() && filename.IsEmpty())
	  return true;

   if (dynamic_cast<TMemoryStream*>(stream) == NULL)
	  return true;

   int error = DB_OK;

   if (!FileName.IsEmpty() && FileName != filename)
	  filename = FileName;

   try {
	((TMemoryStream*)stream)->SaveToFile(filename);
	  modified = false;
   }
   catch(Exception &e)
   {
	  exceptmsg = e.Message;
	  error = DB_WRITEEXC;
	  return error;
   }

   return !error;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TDB::NextStep(int level, const AnsiString &path)
{
   AnsiString token;
   int start = 0, tok = 0, len = path.Length() + 1;
   char div = DBPSc;

   for (int ch = 1; ch <= len; ch++)
   {
      bool isdiv = ch == len || path[ch] == div;
      if (isdiv)
		 tok++;

      if (tok > level)
      {
         token = path.SubString(start + 1, ch - (start + 1));
         break;
      }

	  if (isdiv)
         start = ch;
   }

   return token;
}
//----------------------------------------------------------------------------

void __fastcall TDB::GetMainIndex(TStrings *items)
{
   items->Text = index[0]->Text;
}
//----------------------------------------------------------------------------

TDBindex* __fastcall TDB::GetIndex(int level)
{
   assert(level >= 0);
   assert(level < 16);
   return index[level];
}
//----------------------------------------------------------------------------

void __fastcall TDB::ShowFreeList(const AnsiString &caption)
{
   dbman->ShowFreeList(caption);
}
//---------------------------------------------------------------------------

void __fastcall TDB::ShowDBError(int error)
{
   AnsiString msg;
   switch (error)
   {
      case DB_INITERR: msg = " Database initialization error!   "; break;
      case DB_FLREADERR: msg = " Database freelist read error!   "; break;
      case DB_FLWRITEERR: msg = " Database freelist write error!   "; break;

      case DB_WRITEERR:   msg = " Database write error!   "; break;
      case DB_WRITEEXC:
         msg = " Database write exception!   ";
         msg += "\n " + exceptmsg;
		 break;

      case DB_READERR:    msg = " Database read error!   "; break;
	  case DB_READEXC:
         msg = " Database read exception!   ";
         msg += "\n " + exceptmsg;
         break;

      case DB_SEEKERR:    msg = " Database seek error!   "; break;
      case DB_SEEKEXC:
         msg = " Database seek exception!   ";
         msg += "\n " + exceptmsg;
		 break;

	  case DB_DEFLATEERR:
		 msg = " Database compression error!   ";
		 msg += "\n " + ziperrmsg;
		 break;

	  case DB_INFLATEERR:
		 msg = " Database decompression error!   ";
		 msg += "\n " + ziperrmsg;
		 break;
	  case DB_FILEHEADERERR:
		 return;
	  default:
		 msg = " Unknown database error!   ";
   }

   MessageBox(Screen->ActiveForm->Handle, msg.c_str(), "DB Error", MB_OK);
}

//----------------------------------------------------------------------------
// TDBmanager
//----------------------------------------------------------------------------

__fastcall TDBmanager::TDBmanager(TDB *DB, const AnsiString &Filename)
   : TObject()
{
   db = DB;
   delStream = false;
   stream = db->stream;
   filename = Filename;
   Init();
}
//----------------------------------------------------------------------------

/*__fastcall TDBmanager::TDBmanager(TDB *DB, const AnsiString &Filename)
   : TObject()
{
   db = DB;
   delStream = true;
   stream = new TFileStream(Filename,
      (unsigned short)(FileExists(Filename) ? fmOpenReadWrite : fmCreate));
   filename = Filename;
   Init();
}*/
//----------------------------------------------------------------------------

__fastcall TDBmanager::TDBmanager(TDB *DB, TStream *Stream, const AnsiString &FileName)
   : TObject()
{
   db = DB;
   filename = FileName;
   delStream = false;
   stream = Stream;
   Init();
}
//---------------------------------------------------------------------------

__fastcall TDBmanager::~TDBmanager()
{
   delete freeList;
   if (delStream)
      delete stream;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::CheckDBFileHeader()
{
   int len = StrLen(DBFILEHEADER);
   char *header = new char[len + 1];
   header[len] = '\0';

   stream->Seek(0, soFromBeginning);
   stream->Read(header, len);

   if (AnsiString(DBFILEHEADER) != AnsiString(header))
   {
      delete header;
      AnsiString msg = " The file '" + filename + "' isn't a QS database! ";
	  MessageBox(NULL, msg.c_str(), " FATAL ERROR ", MB_OK);
      error = DB_FILEHEADERERR;
      return false;
   }

   delete [] header;
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TDBmanager::Init()
{
  error = DB_OK;
  startOfData = DBNIL;
  endOfData = StrLen(DBFILEHEADER) + DBFLAGSIZE + (OFSSIZE * 2);
  freeList = new TDBfreeList();

  if(!stream->Size)  // Virgin file; must initialize it.
  {
      if (WriteHeader() && WriteStartAndEnd())
		 WriteFreeList();
  }
  else
  {
      if (CheckDBFileHeader() && ReadStartAndEnd())
         ReadFreeList();
  }
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::WriteHeader()
{
   if (error || !SeekTo(0))
      return false;

   if (!WriteDat(&DBFILEHEADER, StrLen(DBFILEHEADER)) || !WriteDat("  ", 2))
      return false;

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::WriteStartAndEnd()
{
   if (error || !SeekTo(StrLen(DBFILEHEADER) + DBFLAGSIZE))
      return false;

   if (!WriteDat(&startOfData, OFSSIZE * 2))
      return false;

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::ReadStartAndEnd()
{
   if (error || !SeekTo(StrLen(DBFILEHEADER) + DBFLAGSIZE))
      return false;

   if (!ReadDat(&startOfData, OFSSIZE * 2))
	  return false;

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::SeekTo(int offset)
{
   if (error)
      return false;

   try {
      if (stream->Seek(offset, soFromBeginning) != offset)
		 error = DB_SEEKERR;
   }
   catch(Exception &e)
   {
      db->exceptmsg = e.Message;
      error = DB_SEEKEXC;
	  return false;
   }

   return !error;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::WriteDat(void *dat, int size)
{
   if (error)
      return false;

   try {
	  if (stream->Write(dat, size) != size)
         error = DB_WRITEERR;
   }
   catch(Exception &e)
   {
	  db->exceptmsg = e.Message;
      error = DB_WRITEEXC;
      return false;
   }

   return !error;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::ReadDat(void *dat, int size)
{
   if (error)
      return false;

   try {
      if (stream->Read(dat, size) != size)
         error = DB_READERR;
   }
   catch(Exception &e)
   {
      db->exceptmsg = e.Message;
      error = DB_READEXC;
      return false;
   }

   return !error;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::ReadFreeList()
{
   if (error)
      return false;

   if (!SeekTo(endOfData))
      return false;

   if (!freeList->LoadFromStream(stream))
   {
      error = DB_FLREADERR;
      return false;
   }
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::WriteFreeList()
{
  if (error)
     return false;

  if (!SeekTo(endOfData))
     return false;

  if (!freeList->SaveToStream(stream))
  {
	 error = DB_FLWRITEERR;
     return false;
  }

  return true;
}
//---------------------------------------------------------------------------

int __fastcall TDBmanager::Allocate(int spaceRequested)
{
   if (error)
	  return 0;

   if (!spaceRequested)
      return DBNIL;

   // Space we will actually need is space requested plus header:
   int ret, spaceRequired = spaceRequested + HDRSIZE;
   ret = AllocateInFreeList(spaceRequired);
   if (ret == DBNIL)
      ret = AllocateAtEnd(spaceRequired);

   if (error)
      return 0;

   if(startOfData == DBNIL)
   {
      startOfData = ret;
      WriteStartAndEnd();
   }

   if (!error)
      WriteFreeList();

   return ret;
}
//---------------------------------------------------------------------------

int __fastcall TDBmanager::AllocateInFreeList(int spaceRequired)
{
   if (error)
      return 0;

   int startOfAlloc = freeList->Allocate(spaceRequired);
   if (startOfAlloc == DBNIL)
      return DBNIL;	// Unsuccessful

   if (SeekTo(startOfAlloc) && WriteDat(&spaceRequired, LENSIZE))
      WriteFreeList();

   return startOfAlloc + HDRSIZE;
}
//---------------------------------------------------------------------------

int __fastcall TDBmanager::AllocateAtEnd(int spaceRequired)
{
   if(error)
	  return 0;

   int ret = endOfData + HDRSIZE;

   // Record the space allocated in the header:
   if (SeekTo(endOfData) && WriteDat(&spaceRequired, LENSIZE))
   {
      // Update and record new endOfData:
      endOfData += (int)spaceRequired;
      WriteStartAndEnd();
   }

  return ret;
}
//---------------------------------------------------------------------------

void __fastcall TDBmanager::Deallocate(int offset)
{
   if (error)
      return;

   // Can't deallocate first allocated block or FLNIL:
   if ( offset == DBNIL || offset == start() )
      return;

   // zero out space
   int flags, length;
   if (SeekTo(offset) && ReadDat(&flags, FLGSIZE) && ReadDat(&length, LENSIZE))
   {
      int dummy = 0;
      TDBbuffer *buf = new TDBbuffer();
      buf->SetLength(length >= 16 ? 16 : length);
      SeekTo(offset);
      WriteDat(&dummy, sizeof(int)); // flags
      WriteDat(&dummy, sizeof(int)); // length
	  WriteDat(buf->data, buf->length);
      delete buf;
   }

   if (!error)
   {
      // Start of record, including record header:
      int startOfRecord = offset - HDRSIZE;
      int spaceFreed;

      if (SeekTo(startOfRecord) && ReadDat(&spaceFreed, LENSIZE) &&
         AddToFreeList(startOfRecord, spaceFreed))
             WriteFreeList();
   }
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::AddToFreeList(int offset, int space)
{
   if (error)
      return false;

   if (!space)
      return true;

   // If last record is deleted, redefine endOfData.
   if (offset + (int)space >= endOfData)
   {  // If last record removed
      endOfData = offset;
      if (!WriteStartAndEnd())
         return false;
   }
   else
   if (!freeList->EnterFreeSpace(offset, space))
      return false;

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::Decompress(TDBbuffer *buf)
{
   if (error)
      return false;

   int ret;
   unsigned have;
   z_stream strm;

   // allocate inflate state
   strm.zalloc = Z_NULL;
   strm.zfree = Z_NULL;
   strm.opaque = Z_NULL;
   strm.avail_in = 0;
   strm.next_in = Z_NULL;
   ret = inflateInit(&strm);
   if (ret != Z_OK)
   {
      error = DB_INFLATEERR;
      if (strm.msg)
         db->ziperrmsg = strm.msg;
      db->ziperror = ret;

	  return false;
   }

   int outlength = buf->length * 2; // could increase outlength to speed op
   unsigned char *out = new unsigned char[outlength];

   strm.avail_in = buf->length;
   if (strm.avail_in > 0)
   {
      strm.next_in = (unsigned char*)buf->data;
      TMemoryStream *ms = new TMemoryStream();

      // run inflate() on input until output buffer not full
      do {
         strm.avail_out = outlength;
         strm.next_out = out;

         ret = inflate(&strm, Z_NO_FLUSH);
         assert(ret != Z_STREAM_ERROR);  // state not clobbered

         switch (ret)
         {
            case Z_NEED_DICT: ret = Z_DATA_ERROR;     // and fall through
            case Z_DATA_ERROR:
            case Z_MEM_ERROR:
               inflateEnd(&strm);
               error = DB_INFLATEERR;
               if (strm.msg)
                  db->ziperrmsg = strm.msg;
               db->ziperror = ret;
			   return false;
         }

         have = outlength - strm.avail_out;
         ms->Write(out, have);

	  } while (strm.avail_out == 0);

      buf->SetData((char*)ms->Memory, (int)ms->Size); // + 1);
    //buf->data[ms->Size] = '\0';
      delete ms;
   }

   // clean up and return
   inflateEnd(&strm);
   delete [] out;

   bool ok = ret == Z_STREAM_END ? true : false;
   if (!ok)
   {
      error = DB_INFLATEERR;
      if (strm.msg)
		 db->ziperrmsg = strm.msg;
      db->ziperror = ret;
   }
   else
      buf->SetFlag(fCompressed);

    return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TDBmanager::Compress(TDBbuffer *buf)
{
   if (error)
      return false;

   int ret, flush = Z_FINISH;
   unsigned have;
   z_stream strm;

// allocate deflate state
   strm.zalloc = Z_NULL;
   strm.zfree = Z_NULL;
   strm.opaque = Z_NULL;
   ret = deflateInit(&strm, Z_DEFAULT_COMPRESSION);
   if (ret != Z_OK)
   {
      error = DB_DEFLATEERR;
      if (strm.msg)
         db->ziperrmsg = strm.msg;
      db->ziperror = ret;

      return false;
   }

   int outlength = (int)((buf->length * 1.2) + 16);
   TMemoryStream *ms = new TMemoryStream();
   unsigned char *out = new unsigned char[outlength];

   strm.avail_in = buf->length;
   strm.next_in = (unsigned char*)buf->data;

   // run deflate() on input until output buffer not full, finish
   // compression if all of source has been read in
   do {
      strm.avail_out = outlength;
      strm.next_out = out;

      ret = deflate(&strm, flush);    // no bad return value
      assert(ret != Z_STREAM_ERROR);  // state not clobbered

      have = outlength - strm.avail_out;
      ms->Write(out, have);

   } while (strm.avail_out == 0);

   assert(strm.avail_in == 0);   // all input will be used
   assert(ret == Z_STREAM_END);  // stream will be complete
   assert(flush == Z_FINISH);

   buf->SetData((char*)ms->Memory, (int)ms->Size);
   delete out;
   delete ms;

   // clean up and return
   ret = deflateEnd(&strm);
   buf->ClearFlag(fCompressed);

   bool ok = ret == Z_OK;
   if (!ok)
   {
	  error = DB_DEFLATEERR;
      if (strm.msg)
         db->ziperrmsg = strm.msg;
      db->ziperror = ret;
   }

   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TDBmanager::ShowFreeList(const AnsiString &caption)
{
   TStringList *list = new TStringList();
   for (int i = 0; i < freeList->Count(); i++)
      list->Add(IntToStr(freeList->getOffset(i)) + "  " + IntToStr(freeList->getSpace(i)));

   TForm *looker = new TForm((void*)NULL);
   looker->Caption = caption;
   looker->Position = poScreenCenter;
   looker->Width = 300;
   looker->Height = 400;
   looker->BorderStyle = bsSizeable;
   looker->BorderIcons = TBorderIcons() << biSystemMenu;
   TRichEdit *ed = new TRichEdit(looker);
   ed->Parent = looker;
   ed->Align = alClient;
   ed->Font->Name = "Courier New";
   ed->Font->Size = 10;
   ed->PlainText = true;
   ed->ScrollBars = ssBoth;
   ed->WordWrap = false;
   ed->SetTextBuf(list->Text.c_str());

   looker->ShowModal();
   delete ed;
   delete looker;

   delete list;
}

//---------------------------------------------------------------------------
// TDBindex
//---------------------------------------------------------------------------

__fastcall TDBindex::TDBindex(TDBmanager *Fm, AnsiString Name, TDBindex *Parent)
{
   dbm = Fm; // required
   name = Name;
   parent = Parent;

   if (parent && !name.IsEmpty())
      offset = parent->GetOffset(name);
   else
   {
      // initialize first allocation if necessary
      if (dbm->start() == DBNIL)
	  {
         offset = DBNIL;
		 dbm->SeekTo(dbm->Allocate(OFSSIZE));
         dbm->WriteDat(&offset, OFSSIZE);
	  }
      else // or just read offset
      {
         dbm->SeekTo(dbm->start());
         dbm->ReadDat(&offset, OFSSIZE);
      }
   }

   if (offset == DBNIL || dbm->error)
      return;

   TDBbuffer *buf = new TDBbuffer();

   int length;
   dbm->SeekTo(offset);
   dbm->ReadDat(&buf->flags, FLGSIZE);
   dbm->ReadDat(&length, LENSIZE);

   if (!dbm->error)
   {
      buf->SetLength(length);
      dbm->ReadDat(buf->data, buf->length);
      dbm->Decompress(buf);

      if (!dbm->error)
      {
         void *p;
         SetText(buf->c_str());
         for (int i = 0; i < Count; i++)
         {
            dbm->ReadDat(&p, PTRSIZE);
            Objects[i] = (TObject*)p;
         }
	  }
   }

   delete buf;
}
//---------------------------------------------------------------------------

__fastcall TDBindex::~TDBindex()
{
}
//---------------------------------------------------------------------------

void __fastcall TDBindex::ShowIndex()
{
   String idx;
   for (int i = 0; i < Count; i++)
      idx += Strings[i] + "     " + String((int)Objects[i]) + "\n";
   ShowMessage(idx);
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::GetSize(const AnsiString &key)
{
   int ofs = GetOffset(key);
   if (ofs == DBNIL)
      return 0;

   int spc = 0;
   int flags;
   dbm->SeekTo(ofs);
   dbm->ReadDat(&flags, FLGSIZE);
   dbm->ReadDat(&spc, LENSIZE);

   return spc;
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::GetType(const AnsiString &key)
{
   int ofs = GetOffset(key);
   if (ofs == DBNIL)
      return -1;

   int flags;
   dbm->SeekTo(ofs);
   dbm->ReadDat(&flags, FLGSIZE);

   return flags;
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::GetFlags(const AnsiString &key)
{
   int ofs = GetOffset(key);
   if (ofs == DBNIL)
	  return -1;

   int flags;
   dbm->SeekTo(ofs);
   dbm->ReadDat(&flags, FLGSIZE);

   return flags;
}
//---------------------------------------------------------------------------

bool __fastcall TDBindex::IsIndexKey(int pos)//const AnsiString &key)
{
   assert(pos >= 0);
   assert(pos < Count);
   int flags = GetFlags(Strings[pos]);
   return (flags & fIndex) == fIndex;
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::GetOffset(const AnsiString &key)
{
   int p = IndexOf(key);
   return p < 0 ? DBNIL : (int)Objects[p];
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::SetOffset(const AnsiString &key, int Offset)
{
   if (key.IsEmpty())
      return DBNIL;

   int p = IndexOf(key);
   if (p < 0)
   {
      Add(key);
      p = Count - 1;
   }
   Objects[p] = (TObject*)Offset;

   return Offset;
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::PutOffset(const AnsiString &key, int Offset)
{
   int ofs = SetOffset(key, Offset);
   Save();

   return ofs;
}
//---------------------------------------------------------------------------

void __fastcall TDBindex::DelKey(const AnsiString &key) // but not data
{
   int idx = IndexOf(key);
   assert(idx >= 0);

   Delete(idx);
   Save();
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::WriteIndex()
{
   if (offset != DBNIL) // deallocate previous offset to index
      dbm->Deallocate(offset);

   if (dbm->error)
      return 0;

   TDBbuffer *buf = new TDBbuffer(Text.c_str(),Text.Length()+1);
   if (buf)
   {
      buf->SetFlag(fIndex);
	  dbm->Compress(buf);

      if (!dbm->error)
      {
         offset = dbm->Allocate(FLGSIZE + LENSIZE + buf->length + (Count*PTRSIZE));
         dbm->SeekTo(offset);
         dbm->WriteDat(&buf->flags, FLGSIZE);
         dbm->WriteDat(&buf->length, LENSIZE);
         dbm->WriteDat(buf->data, buf->length);

         if (!dbm->error)
         {
            for (int i = 0; i < Count; i++)
			{
               void *p = Objects[i];
               dbm->WriteDat(&p, PTRSIZE);
			}
         }
      }
	  delete buf;
   }
   return offset;
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::Save()
{
   if (dbm->error)
      return 0;

   int ofs = WriteIndex();

   if (parent)
      parent->PutOffset(name, offset);
   else
   {
	  dbm->SeekTo(dbm->start());
	  dbm->WriteDat(&offset, OFSSIZE);
   }

   return ofs;
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::GetBinary(const AnsiString &key, TDBbuffer *buf)
{
   if (dbm->error)
	  return dbm->error;

   if (!buf)
	  return NULL;

   int ofs = GetOffset(key);
   if (ofs == DBNIL)
	  return NULL;

   if (!dbm->error)
   {
	  int length;
	  buf->Clear();
	  dbm->SeekTo(ofs);
	  dbm->ReadDat(&buf->flags, FLGSIZE);
	  dbm->ReadDat(&length, LENSIZE);

	  if (!dbm->error)
	  {
		 buf->SetLength(length);
		 dbm->ReadDat(buf->data, buf->length);
		 dbm->Decompress(buf);
	  }
   }

   return dbm->error; //buf->data;
}
//---------------------------------------------------------------------------

int __fastcall TDBindex::PutBinary(const AnsiString &key, TDBbuffer* buf, bool SaveIndex)
{
   if (dbm->error)
      return dbm->error;

   // get key id or add if not already there
   int pos = IndexOf(key);
   if (pos < 0)
   {
      AddObject(key,(TObject*)DBNIL);
    //pos = Count-1;   this didn't work.. list is sorted. last added might not be last
      pos = IndexOf(key);
   }

   // get offset...deallocate, allocate...write space then data
   int ofs = GetOffset(key);
   if (ofs != DBNIL)
	  dbm->Deallocate(ofs);

   if (dbm->error)
      return dbm->error;

   if (!buf)
      Delete(IndexOf(key));
   else
   {
      buf->ClearFlag(fIndex);
      dbm->Compress(buf);

      if (!dbm->error)
      {
         if ((ofs = dbm->Allocate(FLGSIZE + LENSIZE + buf->length)) != DBNIL)
         {
            dbm->SeekTo(ofs);
            dbm->WriteDat(&buf->flags, FLGSIZE);
			dbm->WriteDat(&buf->length, LENSIZE);
            dbm->WriteDat(buf->data, buf->length);
         }
         // assign new offset to index key and save
         Objects[pos] = (TObject*)ofs;
      }
   }

   if (SaveIndex)
      Save();

   return dbm->error;
}
//---------------------------------------------------------------------------

void __fastcall TDBindex::SwapKey(const AnsiString &msgkey, const AnsiString &newkey)
{
//look(this, msgkey);
//for (int i = 0; i < Count; i++)
//  sm(IntToStr((int)(Strings[i] == msgkey)));
   int idx = IndexOf(msgkey);
   assert(idx >= 0);
   Strings[idx] = newkey;
   Save();
}
//---------------------------------------------------------------------------

void __fastcall TDBindex::FlagByte(AnsiString key, int flagsbyte, byte flag)
{
   int idx = IndexOf(key);
   assert(idx >= 0);
   assert(flagsbyte < key.Length());
   key[flagsbyte] |= flag;
   Strings[idx] = key;
   Save();
}
//---------------------------------------------------------------------------

void __fastcall TDBindex::UnFlagByte(AnsiString key, int flagsbyte, byte flag)
{
   int idx = IndexOf(key);
   assert(idx >= 0);
   assert(flagsbyte < key.Length());

   key[flagsbyte] &= (byte)~flag;
   Strings[idx] = key;
   Save();
}
//---------------------------------------------------------------------------

bool __fastcall TDBindex::ByteIsFlagged(AnsiString key, int flagsbyte, byte flag)
{
   int idx = IndexOf(key);
   assert(idx >= 0);
   assert(flagsbyte < key.Length());

   return (key[flagsbyte] & flag) == flag;
}

//---------------------------------------------------------------------------
// TDBfreeList
//---------------------------------------------------------------------------

__fastcall TDBfreeList::TDBfreeList() : TObject()
{
   offset = new TList();
   space = new TList();
   lastOffset = 0;
   lastSpace = 0;
}
//---------------------------------------------------------------------------

__fastcall TDBfreeList::~TDBfreeList()
{
   delete offset;
   delete space;
}
//---------------------------------------------------------------------------

bool __fastcall TDBfreeList::AddFreeSpace(int Offset, int Space)
{
   int i = 0;
   while (i < offset->Count)
   {
      if (Offset < getOffset(i))
         break;

      i++;
   }
   offset->Insert(i, (void*)Offset);
   lastOffset = Offset;
   space->Insert(i, (void*)Space);
   lastSpace = Space;

   return true;
}
//---------------------------------------------------------------------------

int __fastcall TDBfreeList::Allocate(int Space)
{
   int i = 0;
   while (i < offset->Count)
   {
      if (Space <= getSpace(i))
      {
         int ofs = getOffset(i);
         space->Items[i] = (void*)(getSpace(i) - Space);
         if (space->Items[i] == 0)
         {
            offset->Delete(i);
            space->Delete(i);
         }
         else
            offset->Items[i] = (void*)(getOffset(i) + Space);
         return ofs;
      }

      i++;
   }
   return DBNIL;
}
//---------------------------------------------------------------------------

void __fastcall TDBfreeList::AttemptMerge(int i)
{
   if (i < offset->Count-1)
   {
      if (getOffset(i) + (int)getSpace(i) == getOffset(i + 1))
      {
         space->Items[i] = (void*)(getSpace(i) + getSpace(i + 1));
         offset->Delete(i + 1);
         space->Delete(i + 1);
      }
   }
}
//---------------------------------------------------------------------------

bool __fastcall TDBfreeList::Coalesce(int Offset, int Space)
{
   int i = 0;
   while (i < offset->Count)
   {
      // try merging to right of existing space...
      if (Offset == getOffset(i) + (int)getSpace(i))
      {
         space->Items[i] = (void*)(getSpace(i) + Space);
         AttemptMerge(i);
         return true;
      }

      // try merging to left...
      if (Offset + Space == getOffset(i))
      {
         offset->Items[i] = (void*)Offset;
         lastOffset = Offset;
         space->Items[i] = (void*)(getSpace(i) + Space);
         lastSpace = getSpace(i);
         return true;
      }

      i++;
   }
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall TDBfreeList::EnterFreeSpace(int Offset, int Space)
{
   if (Coalesce(Offset,Space))
      return true;

   return AddFreeSpace(Offset,Space);
}
//---------------------------------------------------------------------------

bool __fastcall TDBfreeList::LoadFromStream(TStream *f)
{
   int i, count, spc, ofs;

   try {
      if (f->Read(&count, INTSIZE) != INTSIZE)
         return false;

      if (offset->Count)
         offset->Clear();

      i = 0;
      while (i < count)
      {
         if (f->Read(&ofs, OFSSIZE) != OFSSIZE)
            return false;

         offset->Add((void*)ofs);
         i++;
      }

      if (space->Count)
         space->Clear();

      i = 0;
      while (i < count)
      {
         if (f->Read(&spc, LENSIZE) != LENSIZE)
            return false;

         space->Add((void*)spc);
         i++;
      }
   }
   catch(...)
   {
      return false;
   }

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TDBfreeList::SaveToStream(TStream *f)
{
   int spc, ofs, i, count = offset->Count;

   try
   {
      if (f->Write(&count, INTSIZE) != INTSIZE)
         return false;

      i = 0;
      while(i < count)
      {
         ofs = getOffset(i);
         if (f->Write(&ofs, OFSSIZE) != OFSSIZE)
            return false;

         i++;
      }

      i = 0;
      while (i < count)
      {
         spc = getSpace(i);
         if (f->Write(&spc, LENSIZE) != LENSIZE)
            return false;

         i++;
      }
   }
   catch(...)
   {
      return false;
   }

   return true;
}
//---------------------------------------------------------------------------

int __fastcall TDBfreeList::Count()
{
   return offset->Count;
}

//---------------------------------------------------------------------------
// TDBbuffer
//---------------------------------------------------------------------------

__fastcall TDBbuffer::TDBbuffer()
   : TObject()
{
   flags = 0;
   data = NULL;
   length = 0;
}
//---------------------------------------------------------------------------

__fastcall TDBbuffer::TDBbuffer(char *Data, int Length)
   : TObject()
{
   assert(Data != NULL);
   if (!Length)
      Length = StrLen(Data);
   Clear();
   SetData(Data, Length);
}
//---------------------------------------------------------------------------

__fastcall TDBbuffer::TDBbuffer(void *Data, int Length)
{
   assert(Data != NULL);
   assert(Length > 0);
   Clear();
   SetData((char*)Data, Length);
}
//---------------------------------------------------------------------------

__fastcall TDBbuffer::TDBbuffer(const AnsiString &Data)
   : TObject()
{
   Clear();
   SetData(Data.c_str(), Data.Length());
}
//---------------------------------------------------------------------------

__fastcall TDBbuffer::~TDBbuffer()
{
   ClearData();
}
//---------------------------------------------------------------------------

void __fastcall TDBbuffer::SetLength(int Length)
{
   ClearData();
   try {
   data = new char[Length + 1];
   }
   catch(...)
   {
	  ShowMessage(Length);
	  exit;
   }
   data[Length] = '\0';
   memset(data, '0', Length);
   length = Length;
}
//---------------------------------------------------------------------------

void __fastcall TDBbuffer::SetData(char *Data, int Length)
{
   ClearData();
   data = new char[Length + 1];
   data[Length] = '\0';
   if (Data)
      memcpy(data, Data, Length);
   length = Length;
}
//---------------------------------------------------------------------------

void __fastcall TDBbuffer::SetData(const AnsiString &Data)
{
   SetData(Data.c_str(), Data.Length());
}
//---------------------------------------------------------------------------

void __fastcall TDBbuffer::ClearFlags()
{
   flags |= fNoflags;
}
//---------------------------------------------------------------------------

void __fastcall TDBbuffer::SetFlag(int flag)
{
   flags |= flag;
}
//---------------------------------------------------------------------------

void __fastcall TDBbuffer::ClearFlag(int flag)
{
   flags &= ~flag;
}
//---------------------------------------------------------------------------

bool __fastcall TDBbuffer::Flagged(int flag)
{
   return (flags & flag) == flag;
}
//---------------------------------------------------------------------------

void __fastcall TDBbuffer::ToggleFlag(int flag)
{
   flags ^= flag;
}
//---------------------------------------------------------------------------

/*bool __fastcall TDBbuffer::Compress()
{
    int ret, flush = Z_FINISH;
    unsigned have;
    z_stream strm;

 // allocate deflate state
    strm.zalloc = Z_NULL;
    strm.zfree = Z_NULL;
    strm.opaque = Z_NULL;
    ret = deflateInit(&strm, Z_DEFAULT_COMPRESSION);
    if (ret != Z_OK)
    {
       if (strm.msg)
          db->ziperrmsg = strm.msg;
       db->ziperror = ret;

       return false;
    }

    int outlength = (length * 1.2) + 16;
    TMemoryStream *ms = new TMemoryStream();
    unsigned char *out = new unsigned char[outlength];

    strm.avail_in = length;
    strm.next_in = (unsigned char*)data;

    // run deflate() on input until output buffer not full, finish
    // compression if all of source has been read in
    do {
        strm.avail_out = outlength;
        strm.next_out = out;

        ret = deflate(&strm, flush);    // no bad return value
        assert(ret != Z_STREAM_ERROR);  // state not clobbered

        have = outlength - strm.avail_out;
        ms->Write(out, have);

    } while (strm.avail_out == 0);

    assert(strm.avail_in == 0);   // all input will be used
    assert(ret == Z_STREAM_END);  // stream will be complete
    assert(flush == Z_FINISH);

    SetData((char*)ms->Memory, (int)ms->Size);
    delete out;
    delete ms;

    // clean up and return
    ret = deflateEnd(&strm);
    ClearFlag(fCompressed);

    bool ok = ret == Z_OK;
    if (!ok)
    {
       if (strm.msg)
          ziperrmsg = strm.msg;
       ziperror = ret;
    }

    return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TDBbuffer::Decompress()
{
    int ret;
    unsigned have;
    z_stream strm;

    // allocate inflate state
    strm.zalloc = Z_NULL;
    strm.zfree = Z_NULL;
    strm.opaque = Z_NULL;
    strm.avail_in = 0;
    strm.next_in = Z_NULL;
    ret = inflateInit(&strm);
    if (ret != Z_OK)
    {
       if (strm.msg)
          ziperrmsg = strm.msg;
       ziperror = ret;

       return false;
    }

    int outlength = length; // * 2; // could increase outlength to speed op
    unsigned char *out = new unsigned char[outlength];

    strm.avail_in = length;
    if (strm.avail_in > 0)
    {
       strm.next_in = (unsigned char*)data;
       TMemoryStream *ms = new TMemoryStream();

       // run inflate() on input until output buffer not full
       do {
          strm.avail_out = outlength;
          strm.next_out = out;

          ret = inflate(&strm, Z_NO_FLUSH);
          assert(ret != Z_STREAM_ERROR);  // state not clobbered

          switch (ret)
          {
             case Z_NEED_DICT:
                ret = Z_DATA_ERROR;     // and fall through
             case Z_DATA_ERROR:
             case Z_MEM_ERROR:
                inflateEnd(&strm);
                return ret;
          }

          have = outlength - strm.avail_out;
          ms->Write(out, have);

       } while (strm.avail_out == 0);

       SetData((char*)ms->Memory, (int)ms->Size);
       delete ms;
    }

    // clean up and return
    inflateEnd(&strm);
    delete [] out;

    bool ok = ret == Z_STREAM_END ? true : false;
    if (!ok)
    {
       if (strm.msg)
          ziperrmsg = strm.msg;
       ziperror = ret;
    }
    else
       SetFlag(fCompressed);

    return ok;
}*/
//---------------------------------------------------------------------------

void __fastcall TDBbuffer::Clear()
{
   flags = 0;
   ClearData();
}
//---------------------------------------------------------------------------

void __fastcall TDBbuffer::ClearData()
{
   if (data)
   {
      ZeroMemory(data, length);
      delete [] data;
      data = NULL;
   }
   length = 0;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TDBbuffer::str()
{
   return data ? AnsiString(data) : AnsiString("");
}
//---------------------------------------------------------------------------

char* __fastcall TDBbuffer::c_str()
{
   return data ? (char*)data : (char*)"\0";
}
//---------------------------------------------------------------------------

