/*
*  eraser.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "eraser.h"
#include "utility.h"
#include "qsadbs.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
HINSTANCE hEraserLib = NULL;
//---------------------------------------------------------------------------

bool __fastcall HasEraser()
{
   return hEraserLib != NULL;
}
//---------------------------------------------------------------------------

AnsiString __fastcall EraserDir()
{
   return AppDir();
}
//---------------------------------------------------------------------------

AnsiString __fastcall EraserDll()
{
   return EraserDir() + "eraser.dll";
}
//---------------------------------------------------------------------------

AnsiString __fastcall EraserVersion()
{
   return "Eraser\t\t" + eraserVersion();
}
//---------------------------------------------------------------------------

bool __fastcall InitEraserLibrary()
{
   if (!FileExists(EraserDll()))
   {
      MsgBox(NULL,
         "Problem, the file eraser.dll is missing from my home directory.  \n"
         "Reinstalling the program will solve the problem.",
          MT_ERROR, MB_OK);
      return false;
   }

   hEraserLib = LoadLibrary(EraserDll().c_str());
   if (!hEraserLib)
   {
      MsgBox(NULL,
         "Sorry, I failed to load eraser.dll. The error reported was:\n\n" +
          GetLastErrorStr(), MT_ERROR, MB_OK);
      return false;
   }

   AnsiString version = eraserVersion();
   if (version == "unknown")
   {
      MsgBox(NULL,
         "The file eraser.dll in my home directory has been replaced with an  \n"
         "unknow version. You must use the eraser.dll that came with QS.  \n"
         "Reinstalling the program will solve the problem.",
          MT_ERROR, MB_OK);
      return false;
   }

   eraserInit();
   return true;
}
//---------------------------------------------------------------------------

void __fastcall FreeEraserLibrary()
{
   if (hEraserLib)
   {
      eraserEnd();
      FreeLibrary(hEraserLib);
   }
}
//---------------------------------------------------------------------------

bool __fastcall eraserWipeFile(ERASER_HANDLE &context, const AnsiString &FileName,
   bool newthread)
{
   TStringList *names = new TStringList();
   names->Add(FileName);
   bool ok = eraserWipeFiles(context, names, newthread);
   delete names;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall eraserWipeFile(const AnsiString &FileName, bool newthread)
{
   ERASER_HANDLE context = NULL;
   return eraserWipeFile(context, FileName, newthread);
}
//---------------------------------------------------------------------------

void __fastcall eraserClearDirectory(const AnsiString &directory, bool newthread)
{
   ERASER_HANDLE context = NULL;
   eraserClearDirectory(context, directory, newthread);
}
//---------------------------------------------------------------------------

void __fastcall eraserClearDirectory(ERASER_HANDLE context,
   const AnsiString &directory, bool newthread)
{
   TStringList *names = new TStringList();
   GetFileNames(names, directory + "*.*");
   if (names->Count)
   {
      eraserWipeFiles(names, newthread);
   }
   delete names;
}
//---------------------------------------------------------------------------

bool __fastcall eraserWipeFiles(TStrings *filenames, bool newthread)
{
   ERASER_HANDLE context = NULL;
   return eraserWipeFiles(context, filenames, newthread);
}
//---------------------------------------------------------------------------

ERASER_METHOD __fastcall GetEraserMethod(const AnsiString &methstr)
{
   if (methstr == "gut") return ERASER_METHOD_GUTMANN;
   if (methstr == "rnd") return ERASER_METHOD_PSEUDORANDOM;
   return ERASER_METHOD_DOD;
}
//---------------------------------------------------------------------------

bool __fastcall eraserWipeFiles(ERASER_HANDLE context, TStrings *filenames,
   bool newthread)
{
   AnsiString errmsg, filename;
   ERASER_RESULT result;
   bool failed;

   //TStringList *cfg = new TStringList();
   //DB->LoadConfig(ERASConfig, cfg);
   bool enabled = hEraserLib != NULL; //ReadBool(cfg, ERASERenabled, ERASEREnabled);
   AnsiString methstr = "dod"; //ReadString(cfg, ERASERwipemethod, ERASERWipeMethod);
   int passes = 7; //ReadInteger(cfg, ERASERpasses, ERASERPasses);
   ERASER_METHOD method = GetEraserMethod(methstr);
   switch (method)
   {
	  case ERASER_METHOD_GUTMANN: passes = 35; break;
	  case ERASER_METHOD_DOD: passes = 7; break;
   }
   //delete cfg;

   if (!enabled) // just delete the files--no wiping desired
   {
	  int attempt = 0;
	  while (filenames->Count)
	  {
		 if (attempt > 0)
			Snooze(500);
		 attempt++;
		 for (int i = 0; i < filenames->Count; i++)
			DeleteFile(filenames->Strings[i]);
		 if (attempt == 10)
			break;
	  }
	  return filenames->Count == 0;
   }

   result = eraserCreateContextEx(&context, (E_UINT16)method, (E_UINT16)passes, 1);
   if (eraserError(result))
   {
	  errmsg = "I failed creating eraser context. Eraser reports: \n\n";
	  goto fail;
   }

   result = eraserSetDataType(context, ERASER_DATA_FILES);
   if (eraserError(result))
   {
      errmsg = "I failed setting eraser data type. Eraser reports: \n\n";
      goto fail;
   }

   failed = false;
   for (int i = 0; i < filenames->Count; i++)
   {
      filename = filenames->Strings[i];
      assert(FileExists(filename));
      result = eraserAddItem(context, filename.c_str(), (E_UINT16)filename.Length());
      if (eraserError(result))
      {
         errmsg = "I failed adding erase filename, " + filename +
            "! Eraser reports: \n\n";
         failed = true;
         break;
      }
   }
   if (failed)
      goto fail;

   if (!newthread)
      result = eraserStartSync(context);
   else
      result = eraserStart(context);

   if (eraserError(result))
   {
      errmsg = "I failed starting eraser thread. Eraser reports: \n\n";
      goto fail;
   }

   if (!newthread)
   {
      eraserDestroyContext(context);
	  //eraserEnd();
   }
   return true;

fail:

   MsgBox(Screen->ActiveForm,
      errmsg + EraserErrorString(result),
      MT_ERROR, MB_OK);
   eraserDestroyContext(context);
   //eraserEnd();

   int i = 0; while (i < filenames->Count)
   {
      if (!FileExists(filenames->Strings[i]))
      {
         filenames->Delete(i);
         continue;
      }
      i++;
   }

   if (!filenames->Count)
      return true;

   Snooze(200);
   failed = false;
   if (MsgBox(Screen->ActiveForm,
      "Eraser failed to remove the following file(s). Shall I just \ngo ahead"
      "and delete them? \n\n" + filenames->Text,
       MT_CONFIRM, MB_YESNO) == IDYES)
   {
      failed = false;
      for (int i = 0; i < filenames->Count; i++)
      {
         filename = filenames->Strings[i];
		 if (!DeleteFile(filename))
         {
            Snooze(200);
            MsgBox(Screen->ActiveForm,
               "Deleting the file, " + filename + ", failed!  \n\n" +
                GetLastErrorStr(),
                MT_ERROR, MB_OK);
            failed = true;
            continue;
         }
      }
   }

   return failed;
}
//---------------------------------------------------------------------------

AnsiString __fastcall EraserErrorString(int error)
{
   switch (error)
   {
      case ERASER_ERROR: return "Unspecified error   ";
      case ERASER_ERROR_PARAM1: return "Parameter 1 invalid   ";
      case ERASER_ERROR_PARAM2: return "Parameter 2 invalid   ";
      case ERASER_ERROR_PARAM3: return "Parameter 3 invalid   ";
      case ERASER_ERROR_PARAM4: return "Parameter 4 invalid   ";
      case ERASER_ERROR_PARAM5: return "Parameter 5 invalid   ";
      case ERASER_ERROR_PARAM6: return "Parameter 6 invalid   ";
      case ERASER_ERROR_MEMORY: return "Out of memory   ";
      case ERASER_ERROR_THREAD: return "Failed to start thread   ";
      case ERASER_ERROR_EXCEPTION: return "BUG!   ";
      case ERASER_ERROR_CONTEXT: return "Context array full   ";
      case ERASER_ERROR_INIT: return "Library not initialized   ";
      case ERASER_ERROR_RUNNING: return "Failed because the thread is running   ";
      case ERASER_ERROR_NOTRUNNING: return "Failed because the thread is not running   ";
      case ERASER_ERROR_DENIED: return "Operation not permitted   ";
	  case ERASER_ERROR_NOTIMPLEMENTED: return "Function not implemented   ";
   }
   return "No error   ";
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserInit()
{
   ERASERINIT eraserinit = (ERASERINIT)GetProcAddress(hEraserLib, "eraserInit");
   if (!eraserinit)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(eraserinit != NULL);
   return eraserinit();
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserEnd()
{
   ERASEREND eraserend = (ERASEREND)GetProcAddress(hEraserLib, "eraserEnd");
   if (!eraserend)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(eraserend != NULL);
   return eraserend();
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserCreateContextEx(E_OUT ERASER_HANDLE* Handle, E_IN E_UINT16 Method,
   E_IN E_UINT16 WParam, E_IN E_UINT8 BParam)
{
   ERASERCREATECONTEXTEX createctx = (ERASERCREATECONTEXTEX)GetProcAddress(hEraserLib, "eraserCreateContextEx");
   if (!createctx)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(createctx != NULL);
   return createctx(Handle, Method, WParam, BParam);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserIsValidContext(E_IN ERASER_HANDLE Handle)
{
   ERASERISVALIDCONTEXT isvalid = (ERASERISVALIDCONTEXT)GetProcAddress(hEraserLib, "eraserIsValidContext");
   if (!isvalid)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(isvalid != NULL);
   return isvalid(Handle);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserDestroyContext(E_IN ERASER_HANDLE Handle)
{
   ERASERDESTROYCONTEXT destroyctx = (ERASERDESTROYCONTEXT)GetProcAddress(hEraserLib, "eraserDestroyContext");
   if (!destroyctx)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(destroyctx != NULL);
   return destroyctx(Handle);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserSetDataType(E_IN ERASER_HANDLE Handle, E_IN E_UINT16 DataType)
{
   ERASERSETDATATYPE settype = (ERASERSETDATATYPE)GetProcAddress(hEraserLib, "eraserSetDataType");
   if (!settype)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(settype != NULL);
   return settype(Handle, DataType);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserAddItem(E_IN ERASER_HANDLE Handle, E_IN LPVOID Item, E_IN E_UINT16 Length)
{
   ERASERADDITEM additem = (ERASERADDITEM)GetProcAddress(hEraserLib, "eraserAddItem");
   if (!additem)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(additem != NULL);
   return additem(Handle, Item, Length);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserStartSync(E_IN ERASER_HANDLE Handle)
{
   ERASERSTARTSYNC startsync = (ERASERSTARTSYNC)GetProcAddress(hEraserLib, "eraserStartSync");
   if (!startsync)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(startsync != NULL);
   return startsync(Handle);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserStart(E_IN ERASER_HANDLE Handle)
{
   ERASERSTART start = (ERASERSTART)GetProcAddress(hEraserLib, "eraserStart");
   if (!start)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(start != NULL);
   return start(Handle);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserStatGetArea(E_IN ERASER_HANDLE Handle, E_OUT E_PUINT64 Value)
{
   ERASERSTATGETAREA getarea = (ERASERSTATGETAREA)GetProcAddress(hEraserLib, "eraserStatGetArea");
   if (!getarea)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(getarea != NULL);
   return getarea(Handle, Value);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserShowReport(E_IN ERASER_HANDLE Handle, E_IN HWND HWnd)
{
   ERASERSHOWREPORT showreport = (ERASERSHOWREPORT)GetProcAddress(hEraserLib, "eraserShowReport");
   if (!showreport)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(showreport);
   return showreport(Handle, HWnd);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserRemoveFile(E_IN LPVOID Name, E_IN E_UINT16 Length)
{
   ERASERREMOVEFILE removefile = (ERASERREMOVEFILE)GetProcAddress(hEraserLib, "eraserRemoveFile");
   if (!removefile)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(removefile != NULL);
   return removefile(Name, Length);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserErrorStringCount(E_IN ERASER_HANDLE Handle, E_OUT E_PUINT16 Count)
{
   ERASERERRORSTRINGCOUNT getcount = (ERASERERRORSTRINGCOUNT)GetProcAddress(hEraserLib, "eraserErrorStringCount");
   if (!getcount)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(getcount != NULL);
   return getcount(Handle, Count);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserErrorString(E_IN ERASER_HANDLE Handle, E_IN E_UINT16 Index,
   E_OUT LPVOID TheString, E_INOUT E_PUINT16 Length)
{
   ERASERERRORSTRING getstring = (ERASERERRORSTRING)GetProcAddress(hEraserLib, "eraserErrorString");
   if (!getstring)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(getstring != NULL);
   return getstring(Handle, Index, TheString, Length);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserProgGetTotalPercent(E_IN ERASER_HANDLE Handle, E_OUT E_PUINT8 Value)
{
   ERASERPROGGETTOTALPERCENT total = (ERASERPROGGETTOTALPERCENT)GetProcAddress(hEraserLib, "eraserProgGetTotalPercent");
   if (!total)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(total != NULL);
   return total(Handle, Value);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserCompleted(E_IN ERASER_HANDLE Handle, E_OUT E_PUINT8 Value)
{
   ERASERCOMPLETED complete = (ERASERCOMPLETED)GetProcAddress(hEraserLib, "eraserCompleted");
   if (!complete)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(complete != NULL);
   return complete(Handle, Value);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserFailed(E_IN ERASER_HANDLE Handle, E_OUT E_PUINT8 Value)
{
   ERASERFAILED failed = (ERASERFAILED)GetProcAddress(hEraserLib, "eraserFailed");
   if (!failed)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(failed != NULL);
   return failed(Handle, Value);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserStop(E_IN ERASER_HANDLE Handle)
{
   ERASERSTOP stop = (ERASERSTOP)GetProcAddress(hEraserLib, "eraserStop");
   if (!stop)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(stop != NULL);
   return stop(Handle);
}
//---------------------------------------------------------------------------

ERASER_EXPORT eraserIsRunning(E_IN ERASER_HANDLE Handle, E_OUT E_PUINT8 Value)
{
   ERASERISRUNNING running = (ERASERISRUNNING)GetProcAddress(hEraserLib, "eraserIsRunning");
   if (!running)
	  MsgBox(Screen->ActiveForm, "ERASER.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(running != NULL);
   return running(Handle, Value);
}
//---------------------------------------------------------------------------

AnsiString eraserVersion()
{
   ERASERVERSION version = (ERASERVERSION)GetProcAddress(hEraserLib, "eraserVersion");
   if (version == NULL)
	  return "unknown";
   return version();
}
//---------------------------------------------------------------------------

