/*
*  gpg4.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver AAM, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver AAM grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "gpg4.h"
#include "global.h"
#include "msgmans.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
const int bufsize = 1024;
bool logop = false;
//---------------------------------------------------------------------------

bool __fastcall GPGGetEncryptionKeys4(TStrings *Message, TStrings *Keys)
{
   Keys->Clear();

   if (!checkgpgavailable())
	  return false;

   AnsiString cmdline = getcommandline(GPGlistencryptionkeys, GPGListEncryptionKeysCmd);

   TMemoryStream *input = new TMemoryStream();
   TMemoryStream *output = new TMemoryStream();
   TStringList *block = new TStringList();
   int x = getpgpblock(Message, block);
   //AnsiString blk =
   DecodeQuotedPrintableInArmor(block);
   //if (blk != block->Text)
   //	  block->Text = blk;
   block->SaveToStream(input);
   putpgpblock(Message, block, x); // just put the block back unmodified--no decryption here
   delete block;

   TGPGConsole4 *con = new TGPGConsole4(NULL);
   bool ok = con->Run(cmdline, NULL, input, output);
//sm(con->error->Text+con->status->Text);
   if (ok)
   {
	  AnsiString id, line;
	  for (int i = 0; i < con->status->Count; i++)
	  {
		 line = con->status->Strings[i];
		 if (line.Pos("ENC_TO") > 0)
		 {
			id = GetTok(2, line, ' ');
			assert(id.Length() == 16);
			Keys->Add("0x" + id.SubString(id.Length() - 7, 8));
		 }
	  }
	  for (int i = 0; i < con->error->Count; i++)
	  {
		 line = con->error->Strings[i];
		 if (line.Pos("passphrase") > 0)
			Keys->Add(line.SubString(20, 64).Trim());
	  }
	  //fix messages from dizum.nl
	  if (!Keys->Count && con->error->Text.Pos(":encrypted data packet:"))
		 Keys->Add("1 passphrase");

	  if (!Keys->Count)
	  {
		 Snooze(200);
		 ShowGPGError(Screen->ActiveForm, con->error, con->status);
		 ok = false;
	  }
   }

   delete con;
   delete output;
   delete input;

//sm(AnsiString("keys ("+IntToStr(Keys->Count)+") :"+Keys->CommaText));
   Snooze(200);
   return ok;
}
//---------------------------------------------------------------------------

/*bool __fastcall GPGGetEncryptionKeys4(TStrings *Message, TStrings *Keys)
{
   Keys->Clear();

   if (!checkgpgavailable())
	  return false;

   AnsiString cmdline = getcommandline(GPGlistencryptionkeys, GPGListEncryptionKeysCmd);

   TMemoryStream *input = new TMemoryStream();
   TMemoryStream *output = new TMemoryStream();
   TStringList *headers = new TStringList();
   getheaders(headers, Message);
   Message->SaveToStream(input);

   TGPGConsole4 *con = new TGPGConsole4(NULL);
   bool ok = con->Run(cmdline, NULL, input, output);
   if (ok)
   {
	  AnsiString id, line;
	  for (int i = 0; i < con->status->Count; i++)
	  {
		 line = con->status->Strings[i];
		 if (line.Pos("ENC_TO") > 0)
		 {
			id = GetTok(2, line, ' ');
			assert(id.Length() == 16);
			Keys->Add("0x" + id.SubString(id.Length() - 7, 8));
		 }
	  }
	  for (int i = 0; i < con->error->Count; i++)
	  {
		 line = con->error->Strings[i];
		 if (line.Pos("passphrase") > 0)
			Keys->Add(line.SubString(20, 64).Trim());
	  }

	  if (!Keys->Count)
	  {
		 Snooze(200);
		 ShowGPGError(Screen->ActiveForm, con->error, con->status);
	  }
   }
   putheaders(Message, headers);

   delete con;
   delete headers;
   delete output;
   delete input;

   Snooze(200);
   return ok;
}*/
//---------------------------------------------------------------------------

bool __fastcall GPGIsFileEncrypted(const AnsiString &FileName)
{
   if (!FileExists(FileName))
	  return false;

   if (!checkgpgavailable())
	  return false;

   AnsiString cmdline = getcommandline(GPGlistencryptionkeysfromfile,
	  GPGListEncryptionKeysFromFileCmd);
   setfield("%filename%", GetShortName(FileName), cmdline);

   TMemoryStream *output = new TMemoryStream();
   TGPGConsole4 *con = new TGPGConsole4(Screen->ActiveForm);
   bool ok = con->Run(cmdline, NULL, NULL, output);
   if (ok)
	  ok = con->error->Text.LowerCase().Pos("no valid openpgp data found") == 0;
   delete con;
   delete output;

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall GPGExportKey(TForm *Owner, const AnsiString keyid, TStrings *keyblock, bool Armor)
{
   TStringList *keyids = new TStringList();
   keyids->Add(keyid);
   bool ok = GPGExportKeys(Owner, keyids, keyblock, Armor);
   delete keyids;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall GPGExportKeys(TForm *Owner, TStrings *keyids, TStrings *keyblock, bool Armor)
{
   assert(keyids != NULL);
   assert(keyblock != NULL);

   if (!checkgpgavailable())
	  return false;

   keyblock->Clear();

   AnsiString keys = keyids->CommaText;
   int x; while ((x = keys.Pos(",")) > 0)
	  keys[x] = ' ';
   AnsiString cmdline = getcommandline(GPGexportkeys, GPGExportKeysCmd);
   setfield("%armor%", Armor ? "--armor" : "--no-armor", cmdline);
   setfield("%keyids%", keys, cmdline);

   TMemoryStream *output = new TMemoryStream();
   TGPGConsole4 *con = new TGPGConsole4(Owner);
   bool ok = con->Run(cmdline, NULL, NULL, output);
   if (ok)
   {
	  ok = con->error->Text.IsEmpty() && con->status->Text.IsEmpty();
	  if (ok)
	  {
		 keyblock->LoadFromStream(output);
		 ok = keyblock->IndexOf(BeginPGPPublicKeyBlock);
		 if (ok)
			ok = keyblock->IndexOf(EndPGPPublicKeyBlock);
	  }
	  if (!ok)
	  {
		 Snooze(200);
		 ShowGPGError(Screen->ActiveForm, con->error, con->status);
	  }
   }
   delete con;
   delete output;

   return ok;
}
//---------------------------------------------------------------------------

void __fastcall FormatSigVerification(TStrings *verified, TStrings *sig)
{
   assert(verified->Count);
   while (verified->Count && !verified->Strings[0].Pos("Signature made"))
	  verified->Delete(0);
   if (!verified->Count)
	  return;

   int x;
   TStringList *veri = new TStringList();
   veri->Add("*** PGP SIGNATURE VERIFICATION ***");
   for (int i = 0; i < verified->Count; i++)
   {
	  AnsiString line = verified->Strings[i];

	  if (line.Pos("gpg:") == 1)
	  {
		 while ((x = line.Pos("\\xe2")) > 0)
			line.Delete(x, 4);
		 while ((x = line.Pos("\\x80")) > 0)
			line.Delete(x, 4);
		 while ((x = line.Pos("\\x9c")) > 0)
			line.Delete(x, 4);
		 while ((x = line.Pos("\\x9d")) > 0)
			line.Delete(x, 4);
		 veri->Add("*** " + line.SubString(6, 128));
	  }
   }
   veri->Add("*** Verified " + FormatDateTime("MM/DD/YY HH:NN:SS", Now()));
   sig->Text = veri->Text;
   delete veri;
}
//---------------------------------------------------------------------------

int __fastcall getpgpblock(TStrings *msg, TStrings *blk)
{
   blk->Clear();
   int x = msg->IndexOf(BeginPGPMessage);
   if (x < 0)
	  x = msg->IndexOf(BeginPGPSignedMessage);
   assert(x >= 0);

   while (msg->Count)
   {
	  AnsiString line = msg->Strings[x];
	  msg->Delete(x);
	  if (line == "**")
		 continue;

	  blk->Add(line);
	  if (line == EndPGPMessage || line == EndPGPSignature)
		 break;
   }
   return x;
}
//---------------------------------------------------------------------------

void __fastcall putpgpblock(TStrings *msg, TStrings *blk, int x)
{
   assert(x <= msg->Count);
   TrimNulls(blk);
   for (int i = 0; i < blk->Count; i++)
   {
	  msg->Insert(x + i, blk->Strings[i]);
   }
}
//---------------------------------------------------------------------------

bool __fastcall GPGVerifyClearSigned4(TForm *Owner, TStrings *Message)
{
   assert(Message->IndexOf(BeginPGPSignedMessage) >= 0);
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"PGP verifying...");
   AnsiString cmdline = getcommandline(GPGverifyclearsigned, GPGVerifyClearSignedCmd);

   TMemoryStream *input = new TMemoryStream();
   TMemoryStream *output = new TMemoryStream();
   TStringList *block = new TStringList();
   int x = getpgpblock(Message, block);
   //AnsiString blk = DecodeQuotedPrintable(block->Text);  DON'T DO IT IN THIS FUNCTION
   //if (blk != block->Text)
   //	  block->Text = blk;
   block->SaveToStream(input);

   TGPGConsole4 *con = new TGPGConsole4(Owner);
   bool ok = con->Run(cmdline, NULL, input, output);
   if (ok)
   {
	  TStringList *sig = new TStringList();
	  FormatSigVerification(con->error, sig);
	  assert(sig->Count);
	  block->LoadFromStream(output);
	  //TrimNulls(block); // don't trim for clear-signed
	  if (!block->Strings[0].Trim().IsEmpty())
		 block->Insert(0, "");
	  block->Insert(0, "*** BEGIN PGP VERIFIED MESSAGE ***");
	  for (int i = 0; i < sig->Count; i++)
		 block->Insert(i, sig->Strings[i]);
	  if (!block->Strings[block->Count-1].Trim().IsEmpty())
		 block->Add("");
	  block->Add("*** END PGP VERIFIED MESSAGE ***");
	  delete sig;
   }
   else
   {
	  Snooze(200);
	  ShowGPGError(Owner, con->error, con->status);
   }
   putpgpblock(Message, block, x);
   delete con;

   delete block;
   delete output;
   delete input;

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Done!");
   Snooze(200);
   return ok;
}
//---------------------------------------------------------------------------

/*AnsiString __fastcall DecodeQuotedPrintableArmor(const AnsiString &text)
{
   int j, i = 1;
   AnsiString decoded, goods = "0123456789ABCDEFabcdef";
   while (i <= text.Length())
   {
	  j = i;
	  while (i <= text.Length() && text[i] != '=')
		 i++;
	  decoded += text.SubString(j, i - j);
	  if (i >= text.Length())
		 break;
	  if (text[i + 1] != '\r' && text[i + 1] != '\n')
	  {
		 AnsiString chrs = text.SubString(i + 1, 2);
		 if (goods.Pos(chrs[1]) && goods.Pos(chrs[2]))
			decoded += AnsiChar(StrToInt("$"+chrs));
		 else
			decoded += text.SubString(i, 3);
	  }
	  i += 3;
   }
   return decoded;
}*/
//---------------------------------------------------------------------------

void __fastcall DecodeQuotedPrintableInArmor(TStrings *msg)
{
   bool gotbegin = false;
   for (int i = 0; i < msg->Count; i++)
   {
	  AnsiString line = msg->Strings[i];
	  // don't decode the last line in the block before end marker
	  if (i < msg->Count-1 && msg->Strings[i + 1] == EndPGPMessage)
	  {
		 if (line.Pos("=3D") == 1)
		 {
			line.Delete(2, 2);
			msg->Strings[i] = line;
		 }
		 break;
	  }
	  if (line == EndPGPMessage)
		 break;
	  if (line == BeginPGPMessage)
		 gotbegin = true;
	  if (!gotbegin)
		 continue;
	  int x = line.Pos("=");
	  if (x > 0 && x < line.Length())
		 msg->Strings[i] = DecodeQuotedPrintable(line);
   }
}
//---------------------------------------------------------------------------

/*void __fastcall DecodeQuotedPrintableInArmor(TStrings *msg)
{
   AnsiString line = msg->Text;
   int x; while ((x = line.Pos("=3D")) > 0)
	  line.Delete(x + 1, 2);
   if (line != msg->Text)
	  msg->Text = line;

   bool gotbegin = false;
   for (int i = 0; i < msg->Count; i++)
   {
	  line = msg->Strings[i];
	  if (i < msg->Count-1 && msg->Strings[i + 1] == EndPGPMessage)
		 break;
	  if (line == EndPGPMessage)
		 break;
	  if (line == BeginPGPMessage)
		 gotbegin = true;
	  if (!gotbegin)
		 continue;

	  x = line.Pos("=");
	  if (x > 0 && x < line.Length())
		 msg->Strings[i] = DecodeQuotedPrintable(line);
   }
}*/
//---------------------------------------------------------------------------

bool __fastcall GPGDecryptMessage4(TForm *Owner, TStrings *Message,
   AnsiString &Passphrase, bool Quiet, TStrings *feedback)
{
   assert(Message->IndexOf(BeginPGPMessage) >= 0);
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"PGP decrypting...");
   AnsiString cmdline = getcommandline(GPGdecrypt, GPGDecryptCmd);
   TMemoryStream *input = new TMemoryStream();
   TMemoryStream *output = new TMemoryStream();
   TStringList *block = new TStringList();
   int x = getpgpblock(Message, block);
   //AnsiString blk =
   DecodeQuotedPrintableInArmor(block);
   //if (blk != block->Text)
   //	  block->Text = blk;
   block->SaveToStream(input);

   TGPGConsole4 *con = new TGPGConsole4(Owner);
   bool ok = con->Run(cmdline, Passphrase.c_str(), input, output);

//block->LoadFromStream(output);
//look(block);
//block->Clear();
//output->Position = 0;

//TStringList *tmp = new TStringList();
//tmp->AddStrings(con->error);
//tmp->AddStrings(con->status);
//look(tmp);
//delete tmp;

   if (con->status->Text.Pos("DECRYPTION_OKAY") && !con->status->Text.Pos("NODATA"))
   {
	  TStringList *sig = new TStringList();
	  FormatSigVerification(con->error, sig);
	  block->LoadFromStream(output);
	  TrimNulls(block);
	  if (sig->Count)
	  {
		 block->Insert(0, "*** BEGIN PGP DECRYPTED/VERIFIED MESSAGE ***");
		 block->Insert(1, "");
		 for (int i = 0; i < sig->Count; i++)
			block->Insert(i, sig->Strings[i]);
		 block->Add("");
		 block->Add("*** END PGP DECRYPTED/VERIFIED MESSAGE ***");
	  }
	  else
	  {
		 block->Insert(0, "*** BEGIN PGP DECRYPTED MESSAGE ***");
		 block->Insert(1, "");
		 block->Add("");
		 block->Add("*** END PGP DECRYPTED MESSAGE ***");
	  }
	  delete sig;
	  ok = true;
   }
   else
   {
	  if (!Quiet)
	  {
		 Snooze(200);
		 ShowGPGError(Owner, con->error, con->status);
	  }
	  ok = false;
   }
   if (feedback)
   {
	  feedback->Clear();
	  feedback->AddStrings(con->error);
	  feedback->Add("-----");
	  feedback->AddStrings(con->status);
   }
   putpgpblock(Message, block, x);
   delete con;

   delete block;
   delete output;
   delete input;
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Done!");
   Snooze(200);
   return ok;
}
//---------------------------------------------------------------------------

/*bool __fastcall GPGDecryptMessage4(TForm *Owner, TStrings *Message,
   AnsiString &Passphrase, bool Quiet, TStrings *Feedback)
{
   assert(Message->IndexOf(BeginPGPMessage) >= 0);
   AnsiString cmdline = getcommandline(GPGdecrypt, GPGDecryptCmd);
   TMemoryStream *input = new TMemoryStream();
   TMemoryStream *output = new TMemoryStream();
   TStringList *block = new TStringList();
   TStringList *headers = new TStringList();

   getheaders(headers, Message);
   assert(Message->Count > 1);
   if (Message->Strings[0] == "")
	  Message->Delete(0);
   if (Message->Count && Message->Strings[0] == "**")
	  Message->Delete(0);
   Message->SaveToStream(input);

   TGPGConsole4 *con = new TGPGConsole4(Owner);
   bool ok = con->Run(cmdline, Passphrase.c_str(), input, output);
   if (con->status->Text.Pos("DECRYPTION_OKAY") && !con->status->Text.Pos("NODATA"))
   {
	  FormatSigVerification(con->error);
	  //Message->AddStrings(headers);
	  if (con->error->Count)
	  {
		 headers->AddStrings(con->error);
		 headers->Add("*** BEGIN PGP DECRYPTED/VERIFIED MESSAGE ***");
		 headers->Add("");
	  }
	  else
	  {
		 headers->Add("*** BEGIN PGP DECRYPTED MESSAGE ***");
		 headers->Add("");
	  }
	  Message->LoadFromStream(output);
	  while (Message->Count &&Message->Strings[Message->Count - 1].Trim().IsEmpty())
		 Message->Delete(Message->Count - 1);
	  while (Message->Count && Message->Strings[0].Trim().IsEmpty())
		 Message->Delete(0);
	  putheaders(Message, headers);
	  if (con->error->Count)
	  {
		 //if (Message->Strings[Message->Count-1].Trim().IsEmpty())
		 Message->Add("");
		 Message->Add("*** END PGP DECRYPTED/VERIFIED MESSAGE ***");
	  }
	  else
	  {
		 //if (Message->Strings[Message->Count-1].Trim().IsEmpty())
		 Message->Add("");
		 Message->Add("*** END PGP DECRYPTED MESSAGE ***");
	  }
	  ok = true;
   }
   else
   {
	  if (!Quiet)
	  {
		 Snooze(200);
		 ShowGPGError(Owner, con->error, con->status);
	  }
	  putheaders(Message, headers);
	  ok = false;
   }
   if (Feedback)
   {
	  Feedback->Clear();
	  Feedback->AddStrings(con->error);
	  Feedback->Add("-----");
	  Feedback->AddStrings(con->status);
   }
   delete con;

   delete headers;
   delete block;
   delete output;
   delete input;

 //Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Done!");
   Snooze(200);
   return ok;
}*/
//---------------------------------------------------------------------------

AnsiString __fastcall GPGGetPublicKeyStr(const AnsiString &Keyid)
{
   assert(Keyid.Length() > 0);
   if (Keyid == "1 passphrase")
	  return "";
   AnsiString keystr;
   TStringList *details = new TStringList();
   GPGListPublicKeysWithColons(details, Keyid);
   for (int i = 0; i < details->Count; i++)
   {
	  if (details->Strings[i].Pos("pub:") == 1)
	  {
		 keystr = details->Strings[i];
		 break;
	  }
   }
   delete details;
   if (!keystr.IsEmpty())
	  keystr = GetTok(9, keystr, ':') + ":" + keystr.SubString(5, keystr.Length());
   return keystr;
}
//---------------------------------------------------------------------------

AnsiString __fastcall GPGGetUserID(const AnsiString &keyid)
{
   return GetTok(lkfKeyUID, GPGGetPublicKeyStr(keyid), ':');
}
//---------------------------------------------------------------------------

bool __fastcall GPGListPublicKeysWithColons(TStrings *Keys,
   const AnsiString &KeyIDs, bool failsilent)
{
   assert(Keys != NULL);

   Keys->Clear();

   if (!checkgpgavailable())
	  return false;

   AnsiString cmdline = getcommandline(GPGlistpublickeyswithcolons,
	  GPGListPublicKeysWithColonsCmd);
   if (!KeyIDs.IsEmpty())
   {
	  AnsiString ids = KeyIDs; // comma separated ?
	  int x = 0; while ((x = ids.Pos(",")) > 0)
		 ids.Delete(x, 1);
	  cmdline += " " + ids.Trim();
   }
   TMemoryStream *output = new TMemoryStream();
   TGPGConsole4 *con = new TGPGConsole4(Screen->ActiveForm);
   bool ok = con->Run(cmdline, NULL, NULL, output);
   if (ok)
   {
	  Keys->LoadFromStream(output);
	  ok = Keys->Count;
	  if (!ok && !failsilent)
	  {
		 Snooze(200);
		 ShowGPGError(Screen->ActiveForm, con->error, con->status);
	  }
   }
   delete con;
   delete output;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall GPGListSecretKeysWithColons(TStrings *keys, const AnsiString &keyids,
   bool failsilent)
{
   assert(keys != NULL);

   keys->Clear();

   if (!checkgpgavailable())
	  return false;

   AnsiString cmdline = getcommandline(GPGlistsecretkeyswithcolons,
	  GPGListSecretKeysWithColonsCmd);

   if (!keyids.IsEmpty())
   {
	  AnsiString ids = keyids; // comma separated?
	  int x = 0; while ((x = ids.Pos(",")) > 0)
		 ids.Delete(x, 1);
	  cmdline += " " + ids.Trim();
   }

   TMemoryStream *output = new TMemoryStream();
   TGPGConsole4 *con = new TGPGConsole4(Screen->ActiveForm);
   bool ok = con->Run(cmdline, NULL, NULL, output);
   if (ok) keys->LoadFromStream(output);
   else
   {
	  if (!failsilent)
	  {
		 Snooze(200);
		 ShowGPGError(Screen->ActiveForm, con->error, con->status);
	  }
   }
   delete con;
   delete output;

   return ok;
}

//---------------------------------------------------------------------------
//   GPGConsole
//---------------------------------------------------------------------------

__fastcall TGPGConsole4::TGPGConsole4(TForm *owner)
   : TObject()
{
   Owner = owner;
   error = new TStringList();
   attrib = new TStringList();
   status = new TStringList();
   statusstream = new TMemoryStream();
   errorstream = new TMemoryStream();
   inputevent = new TEvent(NULL, false, false, "", false);
   log = new TStringList();
}
//---------------------------------------------------------------------------

__fastcall TGPGConsole4::~TGPGConsole4()
{
   delete log;
   delete inputevent;
   delete errorstream;
   delete statusstream;
   delete status;
   delete attrib;
   delete error;
   //DestroyHandles();
}
//---------------------------------------------------------------------------

static void __fastcall CLOSEHandle(HANDLE &handle)
{
   if (handle)
   {
	  CloseHandle(handle);
	  handle = NULL;
   }
}
//---------------------------------------------------------------------------

bool __fastcall TGPGConsole4::CreateHandles(const AnsiString &cmdLine)
{
   TSecurityAttributes sa;
   HANDLE hOutputReadTmp, hErrorReadTmp, hInputWriteTmp, hPassWriteTmp,
	  hAttrReadTmp, hStatReadTmp;

   bool bstatus = cmdLine.Pos("%stat%");
   bool battrib = cmdLine.Pos("%attr%");
   bool bpasswd = cmdLine.Pos("%pass%");

   DestroyHandles();
   sa.nLength = sizeof(sa);
   sa.lpSecurityDescriptor = NULL;
   sa.bInheritHandle = true;

   if (!CreatePipe(&hInputRead, &hInputWriteTmp, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (!CreatePipe(&hOutputReadTmp, &hOutputWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (!CreatePipe(&hErrorReadTmp, &hErrorWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (bstatus && !CreatePipe(&hStatReadTmp, &hStatWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (battrib && !CreatePipe(&hAttrReadTmp, &hAttrWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (bpasswd && !CreatePipe(&hPassRead, &hPassWriteTmp, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (!DuplicateHandle(GetCurrentProcess(), hInputWriteTmp,
	  GetCurrentProcess(), &hInputWrite, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (!DuplicateHandle(GetCurrentProcess(), hOutputReadTmp,
	  GetCurrentProcess(), &hOutputRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (!DuplicateHandle(GetCurrentProcess(), hErrorReadTmp,
	  GetCurrentProcess(), &hErrorRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (bstatus && !DuplicateHandle(GetCurrentProcess(), hStatReadTmp,
	  GetCurrentProcess(), &hStatRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (battrib && !DuplicateHandle(GetCurrentProcess(), hAttrReadTmp,
	  GetCurrentProcess(), &hAttrRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (bpasswd && !DuplicateHandle(GetCurrentProcess(), hPassWriteTmp,
	  GetCurrentProcess(), &hPassWrite, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (battrib) CLOSEHandle(hAttrReadTmp);
   if (bstatus) CLOSEHandle(hStatReadTmp);
   if (bpasswd) CLOSEHandle(hPassWriteTmp);
   CLOSEHandle(hOutputReadTmp);
   CLOSEHandle(hErrorReadTmp);
   CLOSEHandle(hInputWriteTmp);

   return true;
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole4::DestroyHandles()
{
   CLOSEHandle(hInputRead);
   CLOSEHandle(hOutputRead);
   CLOSEHandle(hInputWrite);
   CLOSEHandle(hOutputWrite);
   CLOSEHandle(hErrorRead);
   CLOSEHandle(hErrorWrite);
   CLOSEHandle(hPassRead);
   CLOSEHandle(hPassWrite);
   CLOSEHandle(hAttrRead);
   CLOSEHandle(hAttrWrite);
   CLOSEHandle(hStatRead);
   CLOSEHandle(hStatWrite);
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole4::SetupFileDescriptors(AnsiString &cmdline)
{
   int p;
   AnsiString fd;

   p = cmdline.Pos("%stat%");
   if (p)
   {
	  fd = AnsiString((int)hStatWrite);
	  setfield("%stat%", fd, cmdline);
   }

   p = cmdline.Pos("%attr%");
   if (p)
   {
	  fd = AnsiString((int)hAttrWrite);
	  setfield("%attr%", fd, cmdline);
   }

   p = cmdline.Pos("%pass%");
   if (p)
   {
	  fd = AnsiString((int)hPassRead);
	  setfield("%pass%", fd, cmdline);
   }
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole4::PrepareStartupInfo()
{
   ZeroMemory(&startupinfo, sizeof(STARTUPINFO));
   startupinfo.cb = sizeof(STARTUPINFO);
   startupinfo.lpReserved = NULL;
   startupinfo.lpDesktop = NULL;
   startupinfo.lpTitle = NULL;
   startupinfo.lpReserved2 = NULL;
   startupinfo.cbReserved2 = 0;
   startupinfo.dwFlags = STARTF_USESTDHANDLES;
   startupinfo.hStdInput = hInputRead;
   startupinfo.hStdOutput = hOutputWrite;
   startupinfo.hStdError = hErrorWrite;
}
//---------------------------------------------------------------------------

bool __fastcall TGPGConsole4::LaunchChildProcess(const AnsiString &cmdLine)
{
   if (!CreateProcess(
	  NULL,             // pointer to filename
	  cmdLine.c_str(),  // pointer to command line string
	  NULL, 		    // pointer to process security attributes
	  NULL, 		    // pointer to thread security attributes
	  true,  		    // handle inheritance flag
	  CREATE_NO_WINDOW, // creation flags
	  NULL, 		    // pointer to new environment block
	  NULL, 		    // pointer to current directory name
	 &startupinfo,      // pointer to STARTUPINFO
	 &processinfo       // pointer to PROCESS_INFORMATION
   ))
   {
	  ShowError("Error creating GPG process: \n");
	  return false;
   }

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TGPGConsole4::SendPassphrase(char *passphrase)
{
//sm(passphrase);
   DWORD byteswrote;
   if (!WriteFile(hPassWrite, passphrase, strlen(passphrase), &byteswrote, NULL))
   {
	  ShowError("Error sending the passphrase to GPG: \n");
	  return false;
   }

   CLOSEHandle(hPassWrite);
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole4::Terminate()
{
   if (running)
	  TerminateProcess(processinfo.hProcess, 0);
   terminated = true;
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole4::ShowError(const AnsiString &errormsg)
{
   MsgBox(Owner, errormsg + GetLastErrorStr(), MT_ERROR, MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole4::Log(const AnsiString &thread, int bytes)
{
   AnsiString count = bytes == -1 ? AnsiString("eof") : AnsiString(bytes);
   log->Add(AnsiString(GetTickCount())+"  "+thread+"  "+count);
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole4::Log(const AnsiString &thread, const AnsiString &text)
{
   log->Add(AnsiString(GetTickCount())+"  "+thread+"  "+text);
}
//---------------------------------------------------------------------------

bool __fastcall TGPGConsole4::Run(AnsiString cmdLine, char *Passphrase,
   TStream *Input, TStream *Output)
{
   inputstream = Input;
   outputstream = Output;

   if (!CreateHandles(cmdLine))
	  return false;

   SetupFileDescriptors(cmdLine);
   PrepareStartupInfo();

   if (!LaunchChildProcess(cmdLine))
	  return false;

   if (Passphrase && !SendPassphrase(Passphrase))
	  return false;

   running = true;
   try
   {
	  CLOSEHandle(hOutputWrite);
	  CLOSEHandle(hStatWrite);
	  CLOSEHandle(hErrorWrite);
	  if (inputstream)
	  {
		 //if (logop)
		 //{
		 //	 log->Add(cmdLine);
		 //  log->Add("-------------------------------------");
		 //  log->Add("");
		 //  Log("m", (int)inputstream->Size);
		 //}
		 inputthread = new TStdIOInputThread4(this);
	  }
	  outputthread = new TStdIOOutputThread4(this);
	  statusthread = new TStdIOStatusThread4(this);
	  errorthread = new TStdIOErrorThread4(this);

	  while (MsgWaitForMultipleObjects(1, &processinfo.hProcess, false,
		 INFINITE, QS_ALLINPUT) == WAIT_OBJECT_0 + 1)
			Application->ProcessMessages();
	  while ((inputthread && !inputthread->eof) || !statusthread->eof || !errorthread->eof)
		 Application->ProcessMessages();
   }
   __finally
   {
	  if (!GetExitCodeProcess(processinfo.hProcess, &exitcode))
		 ShowError("Error getting GPG exit code: \n");

	  CloseHandle(processinfo.hThread);
	  CloseHandle(processinfo.hProcess);

	  status->LoadFromStream(statusstream);
	  error->LoadFromStream(errorstream);
//sm(error->Text+status->Text);
	  delete errorthread;
	  delete statusthread;
	  delete outputthread;
	  delete inputthread;
	  DestroyHandles();
	  running = false;
	  if (logop)
	  {
		 log->Add("");
		 log->Add("status-------------------------------");
		 log->AddStrings(status);
		 log->Add("");
		 log->Add("error--------------------------------");
		 log->AddStrings(error);
		 log->Add("");
		 log->Add("output-------------------------------");
		 TStringList *text = new TStringList();
		 outputstream->Position = 0;
		 text->LoadFromStream(outputstream);
		 outputstream->Position = 0;
		 log->AddStrings(text);
		 log->Add("");
		 delete text;
		 //look(log);
	  }
	  return true;
   }
}

//---------------------------------------------------------------------------
// TStdIOInputThread
//---------------------------------------------------------------------------

__fastcall TStdIOInputThread4::TStdIOInputThread4(TGPGConsole4 *Console)
	: TThread(true)
{
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TStdIOInputThread4::Execute()
{
   HANDLE objects[2];
   objects[0] = (void*)con->processinfo.hProcess;
   objects[1] = (void*)con->inputevent->Handle;
   con->inputevent->SetEvent();

   if (WaitForMultipleObjects(2, objects, false, INFINITE) == WAIT_OBJECT_0 + 1)
   {
	  char buffer[bufsize];
	  DWORD bytesWritten, bytesToWrite;
	  TStream *strm = con->inputstream;

	  strm->Seek(0, soFromBeginning);
	  while(strm->Position < strm->Size)
	  {
		 bytesToWrite = strm->Read(buffer, bufsize);
		 if (!WriteFile(con->hInputWrite, buffer, bytesToWrite, &bytesWritten, NULL))
		 {
			con->ShowError("Error sending input to GPG: \n");
			break;
		 }
		 if (logop) con->Log("i", bytesWritten);
		 if (con->terminated)
			break;
	  }
	  FlushFileBuffers(con->hInputWrite);
	  Application->ProcessMessages();
   }

   CLOSEHandle(con->hInputWrite);
   //if (logop) con->Log("i", "eof");
   eof = true;
}

//---------------------------------------------------------------------------
// TStdIOOutputThread
//---------------------------------------------------------------------------

__fastcall TStdIOOutputThread4::TStdIOOutputThread4(TGPGConsole4 *Console)
	: TThread(true)
{
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TStdIOOutputThread4::Execute()
{
   DWORD bytesRead;
   char buffer[bufsize];

   while (!con->terminated && ReadFile(con->hOutputRead, buffer, bufsize, &bytesRead, NULL) && bytesRead > 0)
   {
	  con->outputstream->Write(&buffer[0], bytesRead);
	  if (logop) con->Log("o", bytesRead);
   }
   con->outputstream->Position = 0;
   //CLOSEHandle(con->hOutputRead);
   if (logop) con->Log("o", "eof");
   eof = true;
}

//---------------------------------------------------------------------------
// TStdIOStatusThread
//---------------------------------------------------------------------------

__fastcall TStdIOStatusThread4::TStdIOStatusThread4(TGPGConsole4 *Console)
	: TThread(true)
{
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TStdIOStatusThread4::Execute()
{
   DWORD bytesRead;
   char buffer[bufsize];

   while (!con->terminated && ReadFile(con->hStatRead, buffer, bufsize, &bytesRead, NULL) && bytesRead > 0)
   {
	  con->statusstream->Write(&buffer[0], bytesRead);
	  if (logop) con->Log("s", bytesRead);
   }
   con->statusstream->Position = 0;
   //CLOSEHandle(con->hStatRead);
   if (logop) con->Log("s", "eof");
   eof = true;
}

//---------------------------------------------------------------------------
// TStdIOErrorThread
//---------------------------------------------------------------------------

__fastcall TStdIOErrorThread4::TStdIOErrorThread4(TGPGConsole4 *Console)
	: TThread(true)
{
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TStdIOErrorThread4::Execute()
{
   DWORD bytesRead;
   char buffer[bufsize];

   while (!con->terminated && ReadFile(con->hErrorRead, buffer, bufsize, &bytesRead, NULL) && bytesRead > 0)
   {
	  con->errorstream->Write(&buffer[0], bytesRead);
	  if (logop) con->Log("e", bytesRead);
   }
   con->errorstream->Position = 0;
   //CLOSEHandle(con->hErrorRead);
   if (logop) con->Log("e", "eof");
   eof = true;
}
//---------------------------------------------------------------------------
