/*
*  pgpdpassdlgs2.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "pgpdpassdlgs2.h"
#include "utility.h"
#include "global.h"
const AnsiString selmsgs =
"Please enter a passphrase and I'll attempt to decrypt the the currently selected "
"messages. This could take a few minutes.";
const AnsiString selmsg =
"Please enter a passphrase and I'll attempt to decrypt the current message.";
const AnsiString decmsg =
"Retrieve mail now?";
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPGPDPassDlg2 *PGPDPassDlg2;
//---------------------------------------------------------------------------

AnsiString __fastcall GetSelectedDecryptionPassphrase(TForm *Owner, const AnsiString &passphrase, int keycount)
{
   AnsiString password;
   TPGPDPassDlg2 *pgpdpassdlg = new TPGPDPassDlg2(Owner);
   if (keycount > 1)
	  pgpdpassdlg->Label1->Caption = selmsgs;
   else
	  pgpdpassdlg->Label1->Caption = selmsg;
   if (!passphrase.IsEmpty())
   {
	  pgpdpassdlg->Phrase->Text = passphrase;
	  pgpdpassdlg->Phrase->SelStart = passphrase.Length();
   }
   pgpdpassdlg->ShowModal();
   if (pgpdpassdlg->ModalResult == mrOk)
	  password = pgpdpassdlg->Phrase->Text;
   delete pgpdpassdlg;
   return password;
}
//---------------------------------------------------------------------------

bool __fastcall ConfirmRetrieveMailWithDecryption(TForm *Owner, AnsiString &passphrase, bool &decrypt)
{
   bool ok = false;
   TPGPDPassDlg2 *retrievedlg = new TPGPDPassDlg2(Owner);
   retrievedlg->Caption = " Retrieve AAM Mail";
   retrievedlg->Label1->Caption = decmsg;
   if (passphrase.IsEmpty())
   {
	  retrievedlg->DecryptBox->Visible = true;
	  retrievedlg->DecryptBox->Enabled = true;
	  retrievedlg->DecryptBox->Checked = false;
	  retrievedlg->Phrase->Clear();
	  retrievedlg->Phrase->Enabled = false;
	  retrievedlg->Phrase->ParentColor = true;
	  retrievedlg->HideBox->Enabled = false;
	  retrievedlg->Label2->Enabled = false;
   }
   else
   {
	  retrievedlg->DecryptBox->Visible = true;
	  retrievedlg->DecryptBox->Enabled = true;
	  retrievedlg->DecryptBox->Checked = true;
	  retrievedlg->Phrase->Lines->Text = passphrase;
	  retrievedlg->Phrase->Enabled = true;
	  retrievedlg->Phrase->Color = clWindow;
	  retrievedlg->Phrase->Font->Color = clWindow;
	  retrievedlg->HideBox->Enabled = true;
	  retrievedlg->Label2->Enabled = true;
	  retrievedlg->ActiveControl = retrievedlg->Phrase;
	  retrievedlg->Phrase->SelStart = retrievedlg->Phrase->Text.Length();
   }
   retrievedlg->ShowModal();
   if (retrievedlg->ModalResult == mrOk)// && retrievedlg->DecryptBox->Checked)
   {
	  if (retrievedlg->DecryptBox->Checked)
		 passphrase = retrievedlg->Phrase->Text;
	  ok = true;
   }
   decrypt = retrievedlg->DecryptBox->Checked;
   delete retrievedlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TPGPDPassDlg2::TPGPDPassDlg2(TComponent* Owner)
		: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   Phrase->Clear();
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Phrase->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
   EditColor = Phrase->Font->Color;
   Phrase->Font->Color = clWindow;
   delete ini;
}
//---------------------------------------------------------------------------

__fastcall TPGPDPassDlg2::~TPGPDPassDlg2()
{
   WipeEditBox(Phrase);
}
//---------------------------------------------------------------------------

void __fastcall TPGPDPassDlg2::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrOk && DecryptBox->Checked && Phrase->Text.IsEmpty())
   {
	  MsgBox(this,
		 "You must enter a passphrase--or untick the Decrypt box!",
		  MT_INFO, MB_OK);
	  CanClose = false;
	  return;
   }
   if (ModalResult == mrOk && Phrase->Enabled && Phrase->Text.IsEmpty())
   {
	  MsgBox(this,
		 "You must enter a passphrase--or click Cancel!",
		  MT_INFO, MB_OK);
	  CanClose = false;
	  return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPGPDPassDlg2::HideBoxClick(TObject *Sender)
{
   if (!HideBox->Checked)
	  Phrase->Font->Color = EditColor;
   else
	  Phrase->Font->Color = clWindow;
}
//---------------------------------------------------------------------------

void __fastcall TPGPDPassDlg2::DecryptBoxClick(TObject *Sender)
{
   if (DecryptBox->Checked)
   {
	  Label2->Enabled = true;
	  HideBox->Enabled = true;
	  Phrase->Enabled = true;
	  Phrase->Color = clWindow;
	  ActiveControl = Phrase;
	  Phrase->SelStart = Phrase->Text.Length();
   }
   else
   {
	  Label2->Enabled = false;
	  HideBox->Enabled = false;
	  Phrase->Clear();
	  Phrase->Enabled = false;
	  Phrase->ParentColor = true;
	  ActiveControl = Panel1;
   }
}
//---------------------------------------------------------------------------

